# -*- encoding: utf-8 -*-
import copy
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools import sequencetools
from abjad.tools.topleveltools import new
from abjad.tools.mathtools.BoundedObject import BoundedObject


class Timespan(BoundedObject):
    r'''A timespan.

    ..  container:: example

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(5, 12)
            >>> timespan_3 = timespantools.Timespan(-2, 2)
            >>> timespan_4 = timespantools.Timespan(10, 20)

    Timespans are closed-open intervals.

    Timespans are immutable.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_start_offset',
        '_stop_offset',
        )

    ### INITIALIZER ###

    def __init__(self, start_offset=NegativeInfinity, stop_offset=Infinity):
        BoundedObject.__init__(self)
        start_offset = self._initialize_offset(start_offset)
        stop_offset = self._initialize_offset(stop_offset)
        assert start_offset is not None
        assert stop_offset is not None
        assert start_offset <= stop_offset, repr((start_offset, stop_offset))
        self._start_offset = start_offset
        self._stop_offset = stop_offset

    ### SPECIAL METHODS ###

    def __and__(self, expr):
        r'''Logical AND of two timespans.

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(5, 12)
            >>> timespan_3 = timespantools.Timespan(-2, 2)
            >>> timespan_4 = timespantools.Timespan(10, 20)

        ::

            >>> timespan_1 & timespan_2
            TimespanInventory([Timespan(start_offset=Offset(5, 1), stop_offset=Offset(10, 1))])

        ::

            >>> timespan_1 & timespan_3
            TimespanInventory([Timespan(start_offset=Offset(0, 1), stop_offset=Offset(2, 1))])

        ::

            >>> timespan_1 & timespan_4
            TimespanInventory([])

        ::

            >>> timespan_2 & timespan_3
            TimespanInventory([])

        ::

            >>> timespan_2 & timespan_4
            TimespanInventory([Timespan(start_offset=Offset(10, 1), stop_offset=Offset(12, 1))])

        ::

            >>> timespan_3 & timespan_4
            TimespanInventory([])

        Returns timespan inventory.
        '''
        from abjad.tools import timespantools
        from abjad.tools.topleveltools import new
        expr = self._get_timespan(expr)
        if not self.intersects_timespan(expr):
            return timespantools.TimespanInventory()
        new_start_offset = max(self.start_offset, expr.start_offset)
        new_stop_offset = min(self.stop_offset, expr.stop_offset)
        timespan = new(
            self,
            start_offset=new_start_offset,
            stop_offset=new_stop_offset,
            )
        return timespantools.TimespanInventory([timespan])

    def __eq__(self, timespan):
        r'''Is true when `timespan` is a timespan with equal offsets.

        ::

            >>> timespantools.Timespan(1, 3) == timespantools.Timespan(1, 3)
            True

        Otherwise false:

        ::

            >>> timespantools.Timespan(1, 3) == timespantools.Timespan(2, 3)
            False

        Returns boolean.
        '''
        from abjad.tools import systemtools
        return systemtools.StorageFormatManager.compare(self, timespan)

    def __format__(self, format_specification=''):
        r'''Formats timespan.

        Set `format_specification` to `''` or `'storage'`.

        ::

            >>> print(format(timespan_1))
            timespantools.Timespan(
                start_offset=durationtools.Offset(0, 1),
                stop_offset=durationtools.Offset(10, 1),
                )

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatManager.get_storage_format(self)
        return str(self)

    def __ge__(self, expr):
        r'''Is true when `expr` start offset is greater or equal
        to timespan start offset.

        ::

            >>> timespan_2 >= timespan_3
            True

        Otherwise false:

        ::

            >>> timespan_1 >= timespan_2
            False

        Returns boolean.
        '''
        assert hasattr(expr, 'start_offset'), repr(expr)
        if hasattr(expr, 'stop_offset'):
            if self.start_offset >= expr.start_offset:
                return True
            elif self.start_offset == expr.start_offset and \
                self.stop_offset >= expr.stop_offset:
                return True
            return False
        return self.start_offset >= expr.start_offset

    def __gt__(self, expr):
        r'''Is true when `expr` start offset is greater than
        timespan start offset.

        ::

            >>> timespan_2 > timespan_3
            True

        Otherwise false:

        ::

            >>> timespan_1 > timespan_2
            False

        Returns boolean.
        '''
        assert hasattr(expr, 'start_offset'), repr(expr)
        if hasattr(expr, 'stop_offset'):
            if self.start_offset > expr.start_offset:
                return True
            elif self.start_offset == expr.start_offset and \
                self.stop_offset > expr.stop_offset:
                return True
            return False
        return self.start_offset > expr.start_offset

    def __hash__(self):
        r'''Hashes timespan.

        Required to be explicitely re-defined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(Timespan, self).__hash__()

    def __le__(self, expr):
        r'''Is true when `expr` start offset is less than or equal to
        timespan start offset.

        ::

            >>> timespan_2 <= timespan_3
            False

        Otherwise false:

        ::

            >>> timespan_1 <= timespan_2
            True

        Returns boolean.
        '''
        assert hasattr(expr, 'start_offset'), repr(expr)
        if hasattr(expr, 'stop_offset'):
            if self.start_offset <= expr.start_offset:
                return True
            elif self.start_offset == expr.start_offset and \
                self.stop_offset <= expr.stop_offset:
                return True
            return False
        return self.start_offset <= expr.start_offset

    def __len__(self):
        r'''Defined equal to ``1`` for all timespans.

        ::

            >>> len(timespan_1)
            1

        Returns positive integer.
        '''
        return 1

    def __lt__(self, expr):
        r'''Is true when `expr` start offset is less than timespan start offset.

        ::

            >>> timespan_1 < timespan_2
            True

        Otherwise false:

        ::

            >>> timespan_2 < timespan_3
            False

        Returns boolean.
        '''
        assert hasattr(expr, 'start_offset'), repr(expr)
        if hasattr(expr, 'stop_offset'):
            if self.start_offset < expr.start_offset:
                return True
            elif self.start_offset == expr.start_offset and \
                self.stop_offset < expr.stop_offset:
                return True
            return False
        return self.start_offset < expr.start_offset

    def __ne__(self, timespan):
        r'''Is true when `timespan` is not a timespan with equivalent offsets.

        ::

            >>> timespantools.Timespan(1, 3) != timespantools.Timespan(2, 3)
            True

        Otherwise false:

        ::

            >>> timespantools.Timespan(1, 3) != timespantools.Timespan(2/2, (3, 1))
            False

        Returns boolean.
        '''
        return not self == timespan

    def __or__(self, expr):
        r'''Logical OR of two timespans.

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(5, 12)
            >>> timespan_3 = timespantools.Timespan(-2, 2)
            >>> timespan_4 = timespantools.Timespan(10, 20)

        ::

            >>> new_timespan = timespan_1 | timespan_2
            >>> print(format(new_timespan))
            timespantools.TimespanInventory(
                [
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(0, 1),
                        stop_offset=durationtools.Offset(12, 1),
                        ),
                    ]
                )

        ::

            >>> new_timespan = timespan_1 | timespan_3
            >>> print(format(new_timespan))
            timespantools.TimespanInventory(
                [
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(-2, 1),
                        stop_offset=durationtools.Offset(10, 1),
                        ),
                    ]
                )

        ::


            >>> new_timespan = timespan_1 | timespan_4
            >>> print(format(new_timespan))
            timespantools.TimespanInventory(
                [
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(0, 1),
                        stop_offset=durationtools.Offset(20, 1),
                        ),
                    ]
                )

        ::

            >>> new_timespan = timespan_2 | timespan_3
            >>> print(format(new_timespan))
            timespantools.TimespanInventory(
                [
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(-2, 1),
                        stop_offset=durationtools.Offset(2, 1),
                        ),
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(5, 1),
                        stop_offset=durationtools.Offset(12, 1),
                        ),
                    ]
                )

        ::

            >>> new_timespan = timespan_2 | timespan_4
            >>> print(format(new_timespan))
            timespantools.TimespanInventory(
                [
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(5, 1),
                        stop_offset=durationtools.Offset(20, 1),
                        ),
                    ]
                )

        ::

            >>> new_timespan = timespan_3 | timespan_4
            >>> print(format(new_timespan))
            timespantools.TimespanInventory(
                [
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(-2, 1),
                        stop_offset=durationtools.Offset(2, 1),
                        ),
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(10, 1),
                        stop_offset=durationtools.Offset(20, 1),
                        ),
                    ]
                )

        Returns timespan inventory.
        '''
        from abjad.tools import timespantools
        expr = self._get_timespan(expr)
        if not self.intersects_timespan(expr) and \
            not self.is_tangent_to_timespan(expr):
            result = timespantools.TimespanInventory([self, expr])
            result.sort()
            return result
        new_start_offset = min(self.start_offset, expr.start_offset)
        new_stop_offset = max(self.stop_offset, expr.stop_offset)
        timespan = new(self,
            start_offset=new_start_offset,
            stop_offset=new_stop_offset,
            )
        return timespantools.TimespanInventory([timespan])

    def __sub__(self, expr):
        r'''Subtract `expr` from timespan.

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(5, 12)
            >>> timespan_3 = timespantools.Timespan(-2, 2)
            >>> timespan_4 = timespantools.Timespan(10, 20)

        ::

            >>> timespan_1 - timespan_1
            TimespanInventory([])

        ::

            >>> timespan_1 - timespan_2
            TimespanInventory([Timespan(start_offset=Offset(0, 1), stop_offset=Offset(5, 1))])

        ::

            >>> timespan_1 - timespan_3
            TimespanInventory([Timespan(start_offset=Offset(2, 1), stop_offset=Offset(10, 1))])

        ::

            >>> timespan_1 - timespan_4
            TimespanInventory([Timespan(start_offset=Offset(0, 1), stop_offset=Offset(10, 1))])

        ::

            >>> timespan_2 - timespan_1
            TimespanInventory([Timespan(start_offset=Offset(10, 1), stop_offset=Offset(12, 1))])

        ::

            >>> timespan_2 - timespan_2
            TimespanInventory([])

        ::

            >>> timespan_2 - timespan_3
            TimespanInventory([Timespan(start_offset=Offset(5, 1), stop_offset=Offset(12, 1))])

        ::

            >>> timespan_2 - timespan_4
            TimespanInventory([Timespan(start_offset=Offset(5, 1), stop_offset=Offset(10, 1))])

        ::

            >>> timespan_3 - timespan_3
            TimespanInventory([])

        ::

            >>> timespan_3 - timespan_1
            TimespanInventory([Timespan(start_offset=Offset(-2, 1), stop_offset=Offset(0, 1))])

        ::

            >>> timespan_3 - timespan_2
            TimespanInventory([Timespan(start_offset=Offset(-2, 1), stop_offset=Offset(2, 1))])

        ::

            >>> timespan_3 - timespan_4
            TimespanInventory([Timespan(start_offset=Offset(-2, 1), stop_offset=Offset(2, 1))])

        ::

            >>> timespan_4 - timespan_4
            TimespanInventory([])

        ::

            >>> timespan_4 - timespan_1
            TimespanInventory([Timespan(start_offset=Offset(10, 1), stop_offset=Offset(20, 1))])

        ::

            >>> timespan_4 - timespan_2
            TimespanInventory([Timespan(start_offset=Offset(12, 1), stop_offset=Offset(20, 1))])

        ::

            >>> timespan_4 - timespan_3
            TimespanInventory([Timespan(start_offset=Offset(10, 1), stop_offset=Offset(20, 1))])

        Returns timespan inventory.
        '''
        from abjad.tools import timespantools
        expr = self._get_timespan(expr)
        inventory = timespantools.TimespanInventory()
        if not self.intersects_timespan(expr):
            inventory.append(copy.deepcopy(self))
        elif expr.trisects_timespan(self):
            new_start_offset = self.start_offset
            new_stop_offset = expr.start_offset
            timespan = new(self,
                start_offset=new_start_offset,
                stop_offset=new_stop_offset,
                )
            inventory.append(timespan)
            new_start_offset = expr.stop_offset
            new_stop_offset = self.stop_offset
            timespan = new(self,
                start_offset=new_start_offset,
                stop_offset=new_stop_offset,
                )
            inventory.append(timespan)
        elif expr.contains_timespan_improperly(self):
            pass
        elif expr.overlaps_only_start_of_timespan(self):
            new_start_offset = expr.stop_offset
            new_stop_offset = self.stop_offset
            timespan = new(self,
                start_offset=new_start_offset,
                stop_offset=new_stop_offset,
                )
            inventory.append(timespan)
        elif expr.overlaps_only_stop_of_timespan(self):
            new_start_offset = self.start_offset
            new_stop_offset = expr.start_offset
            timespan = new(self,
                start_offset=new_start_offset,
                stop_offset=new_stop_offset,
                )
            inventory.append(timespan)
        elif expr.starts_when_timespan_starts(self) and \
            expr.stops_before_timespan_stops(self):
            new_start_offset = expr.stop_offset
            new_stop_offset = self.stop_offset
            timespan = new(self,
                start_offset=new_start_offset,
                stop_offset=new_stop_offset,
                )
            inventory.append(timespan)
        elif expr.stops_when_timespan_stops(self) and \
            expr.starts_after_timespan_starts(self):
            new_start_offset = self.start_offset
            new_stop_offset = expr.start_offset
            timespan = new(self,
                start_offset=new_start_offset,
                stop_offset=new_stop_offset,
                )
            inventory.append(timespan)
        else:
            raise ValueError(self, expr)
        return inventory

    def __xor__(self, expr):
        r'''Logical XOR of two timespans.

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(5, 12)
            >>> timespan_3 = timespantools.Timespan(-2, 2)
            >>> timespan_4 = timespantools.Timespan(10, 20)

        ::

            >>> new_timespan = timespan_1 ^ timespan_2
            >>> print(format(new_timespan))
            timespantools.TimespanInventory(
                [
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(0, 1),
                        stop_offset=durationtools.Offset(5, 1),
                        ),
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(10, 1),
                        stop_offset=durationtools.Offset(12, 1),
                        ),
                    ]
                )

        ::

            >>> new_timespan = timespan_1 ^ timespan_3
            >>> print(format(new_timespan))
            timespantools.TimespanInventory(
                [
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(-2, 1),
                        stop_offset=durationtools.Offset(0, 1),
                        ),
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(2, 1),
                        stop_offset=durationtools.Offset(10, 1),
                        ),
                    ]
                )

        ::

            >>> new_timespan = timespan_1 ^ timespan_4
            >>> print(format(new_timespan))
            timespantools.TimespanInventory(
                [
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(0, 1),
                        stop_offset=durationtools.Offset(10, 1),
                        ),
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(10, 1),
                        stop_offset=durationtools.Offset(20, 1),
                        ),
                    ]
                )

        ::

            >>> new_timespan = timespan_2 ^ timespan_3
            >>> print(format(new_timespan))
            timespantools.TimespanInventory(
                [
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(-2, 1),
                        stop_offset=durationtools.Offset(2, 1),
                        ),
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(5, 1),
                        stop_offset=durationtools.Offset(12, 1),
                        ),
                    ]
                )

        ::

            >>> new_timespan = timespan_2 ^ timespan_4
            >>> print(format(new_timespan))
            timespantools.TimespanInventory(
                [
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(5, 1),
                        stop_offset=durationtools.Offset(10, 1),
                        ),
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(12, 1),
                        stop_offset=durationtools.Offset(20, 1),
                        ),
                    ]
                )

        ::

            >>> new_timespan = timespan_3 ^ timespan_4
            >>> print(format(new_timespan))
            timespantools.TimespanInventory(
                [
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(-2, 1),
                        stop_offset=durationtools.Offset(2, 1),
                        ),
                    timespantools.Timespan(
                        start_offset=durationtools.Offset(10, 1),
                        stop_offset=durationtools.Offset(20, 1),
                        ),
                    ]
                )

        Returns timespan inventory.
        '''
        from abjad.tools import timespantools
        expr = self._get_timespan(expr)
        if not self.intersects_timespan(expr) or \
            self.is_tangent_to_timespan(expr):
            result = timespantools.TimespanInventory()
            result.append(copy.deepcopy(self))
            result.append(copy.deepcopy(expr))
            result.sort()
            return result
        result = timespantools.TimespanInventory()
        start_offsets = [self.start_offset, expr.start_offset]
        stop_offsets = [self.stop_offset, expr.stop_offset]
        start_offsets.sort()
        stop_offsets.sort()
        timespan_1 = new(self,
            start_offset=start_offsets[0],
            stop_offset=start_offsets[1],
            )
        timespan_2 = new(self,
            start_offset=stop_offsets[0],
            stop_offset=stop_offsets[1],
            )
        if timespan_1.is_well_formed:
            result.append(timespan_1)
        if timespan_2.is_well_formed:
            result.append(timespan_2)
        result.sort()
        return result

    ### PRIVATE METHODS ###

    def _can_fuse(self, expr):
        if isinstance(expr, type(self)):
            return self.intersects_timespan(expr) or \
                self.stops_when_timespan_starts(expr)
        return False

    def _get_timespan(self, expr):
        if isinstance(expr, Timespan):
            start_offset, stop_offset = expr.offsets
        elif hasattr(expr, '_get_timespan'):
            start_offset, stop_offset = expr._get_timespan().offsets
        # TODO: remove this branch in favor of the _get_timespan above
        elif hasattr(expr, 'get_timespan'):
            start_offset, stop_offset = expr.get_timespan().offsets
        else:
            start_offset, stop_offset = expr.timespan.offsets
        return new(self,
            start_offset=start_offset,
            stop_offset=stop_offset,
            )

    def _implements_timespan_interface(self, timespan):
        if hasattr(timespan, 'start_offset') and \
            hasattr(timespan, 'stop_offset'):
            return True
        if hasattr(timespan, '_get_timespan'):
            return True
        # TODO: remove this branch in favor of the _get_timespan above
        if hasattr(timespan, 'get_timespan'):
            return True
        if hasattr(timespan, 'timespan'):
            return True
        return False

    def _initialize_offset(self, offset):
        if offset in (NegativeInfinity, Infinity):
            return offset
        else:
            return durationtools.Offset(offset)

    ### PUBLIC PROPERTIES ###

    @property
    def axis(self):
        r'''Arithmetic mean of timespan start- and stop-offsets.

        ::

            >>> timespan_1.axis
            Offset(5, 1)

        Returns offset.
        '''
        return (self.start_offset + self.stop_offset) / 2

    @property
    def duration(self):
        r'''Duration of timespan.

        ::

            >>> timespan_1.duration
            Duration(10, 1)

        Returns duration.
        '''
        return self.stop_offset - self.start_offset

    @property
    def is_closed(self):
        r'''False for all timespans.

        ::

            >>> timespan_1.is_closed
            False

        Returns boolean.
        '''
        return BoundedObject.is_closed.fget(self)

    @property
    def is_half_closed(self):
        r'''True for all timespans.

        ::

            >>> timespan_1.is_half_closed
            True

        Returns boolean.
        '''
        return BoundedObject.is_half_closed.fget(self)

    @property
    def is_half_open(self):
        r'''True for all timespans.

        ::

            >>> timespan_1.is_half_open
            True

        Returns boolean.
        '''
        return BoundedObject.is_half_open.fget(self)

    @property
    def is_left_closed(self):
        r'''True for all timespans.

            >>> timespan_1.is_left_closed
            True

        Returns boolean.
        '''
        return True


    @property
    def is_left_open(self):
        r'''False for all timespans.

            >>> timespan_1.is_left_open
            False

        Returns boolean.
        '''
        return False

    @property
    def is_open(self):
        r'''False for all timespans.

        ::

            >>> timespan_1.is_open
            False

        Returns boolean.
        '''
        return BoundedObject.is_open.fget(self)

    @property
    def is_right_closed(self):
        r'''False for all timespans.

            >>> timespan_1.is_right_closed
            False

        Returns boolean.
        '''
        return False


    @property
    def is_right_open(self):
        r'''True for all timespans.

            >>> timespan_1.is_right_open
            True

        Returns boolean.
        '''
        return True

    @property
    def is_well_formed(self):
        r'''Is true when timespan start offset preceeds timespan stop offset.
        Otherwise false:

        ::

            >>> timespan_1.is_well_formed
            True

        Returns boolean.
        '''
        return self.start_offset < self.stop_offset

    @property
    def offsets(self):
        r'''Timespan offsets.

        ::

            >>> timespan_1.offsets
            (Offset(0, 1), Offset(10, 1))

        Returns offset pair.
        '''
        return self.start_offset, self.stop_offset

    @property
    def start_offset(self):
        r'''Timespan start offset.

        ::

            >>> timespan_1.start_offset
            Offset(0, 1)

        Returns offset.
        '''
        return self._start_offset

    @property
    def stop_offset(self):
        r'''Timespan stop offset.

        ::

            >>> timespan_1.stop_offset
            Offset(10, 1)

        Returns offset.
        '''
        return self._stop_offset

    ### PUBLIC METHODS ###

    def contains_timespan_improperly(self, timespan):
        r'''Is true when timespan contains `timespan` improperly.
        Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(5, 10)

        ::

            >>> timespan_1.contains_timespan_improperly(timespan_1)
            True
            >>> timespan_1.contains_timespan_improperly(timespan_2)
            True
            >>> timespan_2.contains_timespan_improperly(timespan_1)
            False
            >>> timespan_2.contains_timespan_improperly(timespan_2)
            True

        Returns boolean.
        '''
        from abjad.tools import timespantools
        if self._implements_timespan_interface(timespan):
            return timespantools.timespan_2_contains_timespan_1_improperly(
                timespan, self)

    def curtails_timespan(self, timespan):
        r'''Is true when timespan curtails `timespan`. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(5, 10)

        ::

            >>> timespan_1.curtails_timespan(timespan_1)
            False
            >>> timespan_1.curtails_timespan(timespan_2)
            False
            >>> timespan_2.curtails_timespan(timespan_1)
            True
            >>> timespan_2.curtails_timespan(timespan_2)
            False

        Returns boolean.
        '''
        from abjad.tools import timespantools
        if self._implements_timespan_interface(timespan):
            return timespantools.timespan_2_curtails_timespan_1(
                timespan, self)

    def delays_timespan(self, timespan):
        r'''Is true when timespan delays `timespan`. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(5, 15)
            >>> timespan_3 = timespantools.Timespan(10, 20)

        ::

            >>> timespan_1.delays_timespan(timespan_2)
            True
            >>> timespan_2.delays_timespan(timespan_3)
            True

        Returns boolean.
        '''
        from abjad.tools import timespantools
        if self._implements_timespan_interface(timespan):
            return timespantools.timespan_2_delays_timespan_1(
                timespan, self)

    def divide_by_ratio(self, ratio):
        r'''Divides timespan by `ratio`.

        ::

            >>> timespan = timespantools.Timespan((1, 2), (3, 2))

        ::

            >>> for x in timespan.divide_by_ratio((1, 2, 1)):
            ...     x
            Timespan(start_offset=Offset(1, 2), stop_offset=Offset(3, 4))
            Timespan(start_offset=Offset(3, 4), stop_offset=Offset(5, 4))
            Timespan(start_offset=Offset(5, 4), stop_offset=Offset(3, 2))

        Returns tuple of newly constructed timespans.
        '''
        if isinstance(ratio, int):
            ratio = ratio * (1, )
        ratio = mathtools.Ratio(ratio)
        unit_duration = self.duration / sum(ratio)
        part_durations = [numerator * unit_duration for numerator in ratio]
        start_offsets = mathtools.cumulative_sums(
            [self.start_offset] + part_durations,
            start=None,
            )
        offset_pairs = sequencetools.iterate_sequence_nwise(
            start_offsets)
        result = [type(self)(*offset_pair) for offset_pair in offset_pairs]
        return tuple(result)

    def get_overlap_with_timespan(self, timespan):
        '''Gets duration of overlap with `timespan`.

        ::

            >>> timespan_1 = timespantools.Timespan(0, 15)
            >>> timespan_2 = timespantools.Timespan(5, 10)
            >>> timespan_3 = timespantools.Timespan(6, 6)
            >>> timespan_4 = timespantools.Timespan(12, 22)


        ::

            >>> timespan_1.get_overlap_with_timespan(timespan_1)
            Duration(15, 1)

        ::

            >>> timespan_1.get_overlap_with_timespan(timespan_2)
            Duration(5, 1)

        ::

            >>> timespan_1.get_overlap_with_timespan(timespan_3)
            Duration(0, 1)

        ::

            >>> timespan_1.get_overlap_with_timespan(timespan_4)
            Duration(3, 1)

        ::

            >>> timespan_2.get_overlap_with_timespan(timespan_2)
            Duration(5, 1)

        ::

            >>> timespan_2.get_overlap_with_timespan(timespan_3)
            Duration(0, 1)

        ::

            >>> timespan_2.get_overlap_with_timespan(timespan_4)
            Duration(0, 1)

        ::

            >>> timespan_3.get_overlap_with_timespan(timespan_3)
            Duration(0, 1)

        ::

            >>> timespan_3.get_overlap_with_timespan(timespan_4)
            Duration(0, 1)

        ::

            >>> timespan_4.get_overlap_with_timespan(timespan_4)
            Duration(10, 1)


        Returns duration.
        '''
        if self._implements_timespan_interface(timespan):
            result = durationtools.Duration(
                sum(x.duration for x in self & timespan))
            return result

    def happens_during_timespan(self, timespan):
        r'''Is true when timespan happens during `timespan`. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(5, 10)

        ::

            >>> timespan_1.happens_during_timespan(timespan_1)
            True
            >>> timespan_1.happens_during_timespan(timespan_2)
            False
            >>> timespan_2.happens_during_timespan(timespan_1)
            True
            >>> timespan_2.happens_during_timespan(timespan_2)
            True

        Returns boolean.
        '''
        from abjad.tools import timespantools
        if self._implements_timespan_interface(timespan):
            return timespantools.timespan_2_happens_during_timespan_1(
                timespan, self)

    def intersects_timespan(self, timespan):
        r'''Is true when timespan intersects `timespan`. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(5, 15)
            >>> timespan_3 = timespantools.Timespan(10, 15)

        ::

            >>> timespan_1.intersects_timespan(timespan_1)
            True
            >>> timespan_1.intersects_timespan(timespan_2)
            True
            >>> timespan_1.intersects_timespan(timespan_3)
            False

        Returns boolean.
        '''
        from abjad.tools import timespantools
        if self._implements_timespan_interface(timespan):
            return timespantools.timespan_2_intersects_timespan_1(
                timespan, self)

    def is_congruent_to_timespan(self, timespan):
        r'''Is true when timespan is congruent to `timespan`. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(5, 15)

        ::

            >>> timespan_1.is_congruent_to_timespan(timespan_1)
            True
            >>> timespan_1.is_congruent_to_timespan(timespan_2)
            False
            >>> timespan_2.is_congruent_to_timespan(timespan_1)
            False
            >>> timespan_2.is_congruent_to_timespan(timespan_2)
            True

        Returns boolean.
        '''
        from abjad.tools import timespantools
        if self._implements_timespan_interface(timespan):
            return timespantools.timespan_2_is_congruent_to_timespan_1(
                timespan, self)

    def is_tangent_to_timespan(self, timespan):
        r'''Is true when timespan is tangent to `timespan`. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(10, 20)

        ::

            >>> timespan_1.is_tangent_to_timespan(timespan_1)
            False
            >>> timespan_1.is_tangent_to_timespan(timespan_2)
            True
            >>> timespan_2.is_tangent_to_timespan(timespan_1)
            True
            >>> timespan_2.is_tangent_to_timespan(timespan_2)
            False

        Returns boolean.
        '''
        if hasattr(timespan, 'start_offset'):
            if self.stop_offset == timespan.start_offset:
                return True
        if hasattr(timespan, 'stop_offset'):
            if timespan.stop_offset == self.start_offset:
                return True
        return False

    def overlaps_all_of_timespan(self, timespan):
        r'''Is true when timespan overlaps all of `timespan`. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(5, 6)
            >>> timespan_3 = timespantools.Timespan(5, 10)

        ::

            >>> timespan_1.overlaps_all_of_timespan(timespan_1)
            False
            >>> timespan_1.overlaps_all_of_timespan(timespan_2)
            True
            >>> timespan_1.overlaps_all_of_timespan(timespan_3)
            False

        Returns boolean.
        '''
        from abjad.tools import timespantools
        if self._implements_timespan_interface(timespan):
            return timespantools.timespan_2_overlaps_all_of_timespan_1(
                timespan, self)

    def overlaps_only_start_of_timespan(self, timespan):
        r'''Is true when timespan overlaps only start of `timespan`.
        Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(-5, 5)
            >>> timespan_3 = timespantools.Timespan(4, 6)
            >>> timespan_4 = timespantools.Timespan(5, 15)

        ::

            >>> timespan_1.overlaps_only_start_of_timespan(timespan_1)
            False
            >>> timespan_1.overlaps_only_start_of_timespan(timespan_2)
            False
            >>> timespan_1.overlaps_only_start_of_timespan(timespan_3)
            False
            >>> timespan_1.overlaps_only_start_of_timespan(timespan_4)
            True

        Returns boolean.
        '''
        from abjad.tools import timespantools
        if self._implements_timespan_interface(timespan):
            return timespantools.timespan_2_overlaps_only_start_of_timespan_1(
                timespan, self)

    def overlaps_only_stop_of_timespan(self, timespan):
        r'''Is true when timespan overlaps only stop of `timespan`.
        Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(-5, 5)
            >>> timespan_3 = timespantools.Timespan(4, 6)
            >>> timespan_4 = timespantools.Timespan(5, 15)

        ::

            >>> timespan_1.overlaps_only_stop_of_timespan(timespan_1)
            False
            >>> timespan_1.overlaps_only_stop_of_timespan(timespan_2)
            True
            >>> timespan_1.overlaps_only_stop_of_timespan(timespan_3)
            False
            >>> timespan_1.overlaps_only_stop_of_timespan(timespan_4)
            False

        Returns boolean.
        '''
        from abjad.tools import timespantools
        if self._implements_timespan_interface(timespan):
            return timespantools.timespan_2_overlaps_only_stop_of_timespan_1(
                timespan, self)

    def overlaps_start_of_timespan(self, timespan):
        r'''Is true when timespan overlaps start of `timespan`.
        Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(-5, 5)
            >>> timespan_3 = timespantools.Timespan(4, 6)
            >>> timespan_4 = timespantools.Timespan(5, 15)

        ::

            >>> timespan_1.overlaps_start_of_timespan(timespan_1)
            False
            >>> timespan_1.overlaps_start_of_timespan(timespan_2)
            False
            >>> timespan_1.overlaps_start_of_timespan(timespan_3)
            True
            >>> timespan_1.overlaps_start_of_timespan(timespan_4)
            True

        Returns boolean.
        '''
        from abjad.tools import timespantools
        if self._implements_timespan_interface(timespan):
            return timespantools.timespan_2_overlaps_start_of_timespan_1(
                timespan, self)

    def overlaps_stop_of_timespan(self, timespan):
        r'''Is true when timespan overlaps start of `timespan`.
        Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(-5, 5)
            >>> timespan_3 = timespantools.Timespan(4, 6)
            >>> timespan_4 = timespantools.Timespan(5, 15)

        ::

            >>> timespan_1.overlaps_stop_of_timespan(timespan_1)
            False
            >>> timespan_1.overlaps_stop_of_timespan(timespan_2)
            True
            >>> timespan_1.overlaps_stop_of_timespan(timespan_3)
            True
            >>> timespan_1.overlaps_stop_of_timespan(timespan_4)
            False

        Returns boolean.
        '''
        from abjad.tools import timespantools
        if self._implements_timespan_interface(timespan):
            return timespantools.timespan_2_overlaps_stop_of_timespan_1(
                timespan, self)

    def reflect(self, axis=None):
        r'''Reflects timespan about `axis`.

        ..  container:: example

            **Example 1.** Reverse timespan about timespan axis:

            ::

                >>> timespantools.Timespan(3, 6).reflect()
                Timespan(start_offset=Offset(3, 1), stop_offset=Offset(6, 1))

        ..  container:: example

            **Example 2.** Reverse timespan about arbitrary axis:

            ::

                >>> timespantools.Timespan(3, 6).reflect(axis=Offset(10))
                Timespan(start_offset=Offset(14, 1), stop_offset=Offset(17, 1))

        Returns new timespan.
        '''
        if axis is None:
            axis = self.axis
        start_distance = self.start_offset - axis
        stop_distance = self.stop_offset - axis
        new_start_offset = axis - stop_distance
        new_stop_offset = axis - start_distance
        return self.set_offsets(new_start_offset, new_stop_offset)

    def round_offsets(self, multiplier, anchor=Left, must_be_well_formed=True):
        '''Rounds timespan offsets to multiple of `multiplier`.

        ::

            >>> timespan = timespantools.Timespan((1, 5), (4, 5))

        ::

            >>> timespan.round_offsets(1)
            Timespan(start_offset=Offset(0, 1), stop_offset=Offset(1, 1))

        ::

            >>> timespan.round_offsets(2)
            Timespan(start_offset=Offset(0, 1), stop_offset=Offset(2, 1))

        ::

            >>> timespan.round_offsets(
            ...     2,
            ...     anchor=Right,
            ...     )
            Timespan(start_offset=Offset(-2, 1), stop_offset=Offset(0, 1))

        ::

            >>> timespan.round_offsets(
            ...     2,
            ...     anchor=Right,
            ...     must_be_well_formed=False,
            ...     )
            Timespan(start_offset=Offset(0, 1), stop_offset=Offset(0, 1))

        Returns new timespan.
        '''
        from abjad.tools.topleveltools import new
        multiplier = abs(durationtools.Multiplier(multiplier))
        assert 0 < multiplier
        new_start_offset = durationtools.Offset(
            int(round(self.start_offset / multiplier)) * multiplier)
        new_stop_offset = durationtools.Offset(
            int(round(self.stop_offset / multiplier)) * multiplier)
        if (new_start_offset == new_stop_offset) and must_be_well_formed:
            if anchor is Left:
                new_stop_offset = new_stop_offset + multiplier
            else:
                new_start_offset = new_start_offset - multiplier
        result = new(
            self,
            start_offset=new_start_offset,
            stop_offset=new_stop_offset,
            )
        return result

    def scale(self, multiplier, anchor=Left):
        r'''Scales timespan by `multiplier`.

            >>> timespan = timespantools.Timespan(3, 6)

        ..  container:: example

            **Example 1.** Scale timespan relative to timespan start offset:

            ::

                >>> timespan.scale(Multiplier(2))
                Timespan(start_offset=Offset(3, 1), stop_offset=Offset(9, 1))

        ..  container:: example

            **Example 2.** Scale timespan relative to timespan stop offset:

            ::

                >>> timespan.scale(Multiplier(2), anchor=Right)
                Timespan(start_offset=Offset(0, 1), stop_offset=Offset(6, 1))

        Returns new timespan.
        '''
        from abjad.tools.topleveltools import new
        multiplier = durationtools.Multiplier(multiplier)
        assert 0 < multiplier
        new_duration = multiplier * self.duration
        if anchor is Left:
            new_start_offset = self.start_offset
            new_stop_offset = self.start_offset + new_duration
        elif anchor is Right:
            new_stop_offset = self.stop_offset
            new_start_offset = self.stop_offset - new_duration
        else:
            raise ValueError
        result = new(
            self,
            start_offset=new_start_offset,
            stop_offset=new_stop_offset,
            )
        return result

    def set_duration(self, duration):
        r'''Sets timespan duration to `duration`.

        ::

            >>> timespan = timespantools.Timespan((1, 2), (3, 2))

        ::

            >>> timespan.set_duration(Duration(3, 5))
            Timespan(start_offset=Offset(1, 2), stop_offset=Offset(11, 10))

        Returns new timespan.
        '''
        from abjad.tools.topleveltools import new
        duration = durationtools.Duration(duration)
        new_stop_offset = self.start_offset + duration
        result = new(
            self,
            stop_offset=new_stop_offset,
            )
        return result

    def set_offsets(self, start_offset=None, stop_offset=None):
        r'''Sets timespan start offset to `start_offset` and
        stop offset to `stop_offset`.

        ::

            >>> timespan = timespantools.Timespan((1, 2), (3, 2))

        ::

            >>> timespan.set_offsets(stop_offset=Offset(7, 8))
            Timespan(start_offset=Offset(1, 2), stop_offset=Offset(7, 8))

        Subtracts negative `start_offset` from existing stop offset:

        ::

            >>> timespan.set_offsets(start_offset=Offset(-1, 2))
            Timespan(start_offset=Offset(1, 1), stop_offset=Offset(3, 2))

        Subtracts negative `stop_offset` from existing stop offset:

        ::

            >>> timespan.set_offsets(stop_offset=Offset(-1, 2))
            Timespan(start_offset=Offset(1, 2), stop_offset=Offset(1, 1))

        Returns new timespan.
        '''
        from abjad.tools.topleveltools import new
        if start_offset is not None and 0 <= start_offset:
            new_start_offset = start_offset
        elif start_offset is not None and start_offset < 0:
            new_start_offset = \
                self.stop_offset + durationtools.Offset(start_offset)
        else:
            new_start_offset = self.start_offset
        if stop_offset is not None and 0 <= stop_offset:
            new_stop_offset = stop_offset
        elif stop_offset is not None and stop_offset < 0:
            new_stop_offset = \
                self.stop_offset + durationtools.Offset(stop_offset)
        else:
            new_stop_offset = self.stop_offset
        result = new(
            self,
            start_offset=new_start_offset,
            stop_offset=new_stop_offset,
            )
        return result

    # TODO: extend to self.split_at_offsets()
    def split_at_offset(self, offset):
        r'''Split into two parts when `offset` happens during timespan:

        ::

            >>> timespan = timespantools.Timespan(0, 5)

        ::

            >>> left, right = timespan.split_at_offset(Offset(2))

        ::

            >>> left
            Timespan(start_offset=Offset(0, 1), stop_offset=Offset(2, 1))

        ::

            >>> right
            Timespan(start_offset=Offset(2, 1), stop_offset=Offset(5, 1))

        Otherwise return a copy of timespan:

        ::

            >>> timespan.split_at_offset(Offset(12))
            Timespan(start_offset=Offset(0, 1), stop_offset=Offset(5, 1))

        Returns one or two newly constructed timespans.
        '''
        offset = durationtools.Offset(offset)
        if self.start_offset < offset < self.stop_offset:
            left = new(self,
                start_offset=self.start_offset,
                stop_offset=offset,
                )
            right = new(self,
                start_offset=offset,
                stop_offset=self.stop_offset,
                )
            return left, right
        else:
            return new(self)

    def split_at_offsets(self, offsets):
        r'''Split into one or more parts when `offsets` happens during
        timespan:

        ::

            >>> timespan = timespantools.Timespan(0, 10)

        ::

            >>> shards = timespan.split_at_offsets((1, 3, 7))
            >>> for shard in shards:
            ...     shard
            ...
            Timespan(start_offset=Offset(0, 1), stop_offset=Offset(1, 1))
            Timespan(start_offset=Offset(1, 1), stop_offset=Offset(3, 1))
            Timespan(start_offset=Offset(3, 1), stop_offset=Offset(7, 1))
            Timespan(start_offset=Offset(7, 1), stop_offset=Offset(10, 1))

        Otherwise return a tuple containing a copy of timespan:

        ::

            >>> timespan.split_at_offsets((-100,))
            (Timespan(start_offset=Offset(0, 1), stop_offset=Offset(10, 1)),)

        Returns one or more newly constructed timespans.
        '''
        offsets = [durationtools.Offset(offset) for offset in offsets]
        offsets = [offset for offset in offsets
            if self.start_offset < offset < self.stop_offset]
        offsets = sorted(set(offsets))
        results = []
        right = new(self)
        for offset in offsets:
            left, right = right.split_at_offset(offset)
            results.append(left)
        results.append(right)
        return tuple(results)

    def starts_after_offset(self, offset):
        r'''Is true when timespan overlaps start of `timespan`. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)

        ::

            >>> timespan_1.starts_after_offset(Offset(-5))
            True
            >>> timespan_1.starts_after_offset(Offset(0))
            False
            >>> timespan_1.starts_after_offset(Offset(5))
            False

        Returns boolean.
        '''
        offset = durationtools.Offset(offset)
        return offset < self.start_offset

    def starts_after_timespan_starts(self, timespan):
        r'''Is true when timespan starts after `timespan` starts. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(5, 15)

        ::

            >>> timespan_1.starts_after_timespan_starts(timespan_1)
            False
            >>> timespan_1.starts_after_timespan_starts(timespan_2)
            False
            >>> timespan_2.starts_after_timespan_starts(timespan_1)
            True
            >>> timespan_2.starts_after_timespan_starts(timespan_2)
            False

        Returns boolean.
        '''
        from abjad.tools import timespantools
        if self._implements_timespan_interface(timespan):
            return timespantools.timespan_2_starts_after_timespan_1_starts(
                timespan, self)

    def starts_after_timespan_stops(self, timespan):
        r'''Is true when timespan starts after `timespan` stops. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(5, 15)
            >>> timespan_3 = timespantools.Timespan(10, 20)
            >>> timespan_4 = timespantools.Timespan(15, 25)

        ::

            >>> timespan_1.starts_after_timespan_stops(timespan_1)
            False
            >>> timespan_2.starts_after_timespan_stops(timespan_1)
            False
            >>> timespan_3.starts_after_timespan_stops(timespan_1)
            True
            >>> timespan_4.starts_after_timespan_stops(timespan_1)
            True

        Returns boolean.
        '''
        from abjad.tools import timespantools
        if self._implements_timespan_interface(timespan):
            return timespantools.timespan_2_starts_after_timespan_1_stops(
                timespan, self)

    def starts_at_offset(self, offset):
        r'''Is true when timespan starts at `offset`. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)

        ::

            >>> timespan_1.starts_at_offset(Offset(-5))
            False
            >>> timespan_1.starts_at_offset(Offset(0))
            True
            >>> timespan_1.starts_at_offset(Offset(5))
            False

        Returns boolean.
        '''
        offset = durationtools.Offset(offset)
        return self.start_offset == offset

    def starts_at_or_after_offset(self, offset):
        r'''Is true when timespan starts at or after `offset`. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)

        ::

            >>> timespan_1.starts_at_or_after_offset(Offset(-5))
            True
            >>> timespan_1.starts_at_or_after_offset(Offset(0))
            True
            >>> timespan_1.starts_at_or_after_offset(Offset(5))
            False

        Returns boolean.
        '''
        offset = durationtools.Offset(offset)
        return offset <= self.start_offset

    def starts_before_offset(self, offset):
        r'''Is true when timespan starts before `offset`. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)

        ::

            >>> timespan_1.starts_before_offset(Offset(-5))
            False
            >>> timespan_1.starts_before_offset(Offset(0))
            False
            >>> timespan_1.starts_before_offset(Offset(5))
            True

        Returns boolean.
        '''
        offset = durationtools.Offset(offset)
        return self.start_offset < offset

    def starts_before_or_at_offset(self, offset):
        r'''Is true when timespan starts before or at `offset`.
        Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)

        ::

            >>> timespan_1.starts_before_or_at_offset(Offset(-5))
            False
            >>> timespan_1.starts_before_or_at_offset(Offset(0))
            True
            >>> timespan_1.starts_before_or_at_offset(Offset(5))
            True

        Returns boolean.
        '''
        offset = durationtools.Offset(offset)
        return self.start_offset <= offset

    def starts_before_timespan_starts(self, timespan):
        r'''Is true when timespan starts before `timespan` starts.
        Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(5, 15)

        ::

            >>> timespan_1.starts_before_timespan_starts(timespan_1)
            False
            >>> timespan_1.starts_before_timespan_starts(timespan_2)
            True
            >>> timespan_2.starts_before_timespan_starts(timespan_1)
            False
            >>> timespan_2.starts_before_timespan_starts(timespan_2)
            False

        Returns boolean.
        '''
        from abjad.tools import timespantools
        if self._implements_timespan_interface(timespan):
            return timespantools.timespan_2_starts_before_timespan_1_starts(
                timespan, self)

    def starts_before_timespan_stops(self, timespan):
        r'''Is true when timespan starts before `timespan` stops. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(5, 15)

        ::

            >>> timespan_1.starts_before_timespan_stops(timespan_1)
            True
            >>> timespan_1.starts_before_timespan_stops(timespan_2)
            True
            >>> timespan_2.starts_before_timespan_stops(timespan_1)
            True
            >>> timespan_2.starts_before_timespan_stops(timespan_2)
            True

        Returns boolean.
        '''
        from abjad.tools import timespantools
        if self._implements_timespan_interface(timespan):
            return timespantools.timespan_2_starts_before_timespan_1_stops(
                timespan, self)

    def starts_during_timespan(self, timespan):
        r'''Is true when timespan starts during `timespan`. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(5, 15)

        ::

            >>> timespan_1.starts_during_timespan(timespan_1)
            True
            >>> timespan_1.starts_during_timespan(timespan_2)
            False
            >>> timespan_2.starts_during_timespan(timespan_1)
            True
            >>> timespan_2.starts_during_timespan(timespan_2)
            True

        Returns boolean.
        '''
        from abjad.tools import timespantools
        if self._implements_timespan_interface(timespan):
            return timespantools.timespan_2_starts_during_timespan_1(
                timespan, self)

    def starts_when_timespan_starts(self, timespan):
        r'''Is true when timespan starts when `timespan` starts. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(5, 15)

        ::

            >>> timespan_1.starts_when_timespan_starts(timespan_1)
            True
            >>> timespan_1.starts_when_timespan_starts(timespan_2)
            False
            >>> timespan_2.starts_when_timespan_starts(timespan_1)
            False
            >>> timespan_2.starts_when_timespan_starts(timespan_2)
            True

        Returns boolean.
        '''
        from abjad.tools import timespantools
        if self._implements_timespan_interface(timespan):
            return timespantools.timespan_2_starts_when_timespan_1_starts(
                timespan, self)

    def starts_when_timespan_stops(self, timespan):
        r'''Is true when timespan starts when `timespan` stops. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(10, 20)

        ::

            >>> timespan_1.starts_when_timespan_stops(timespan_1)
            False
            >>> timespan_1.starts_when_timespan_stops(timespan_2)
            False
            >>> timespan_2.starts_when_timespan_stops(timespan_1)
            True
            >>> timespan_2.starts_when_timespan_stops(timespan_2)
            False

        Returns boolean.
        '''
        from abjad.tools import timespantools
        if self._implements_timespan_interface(timespan):
            return timespantools.timespan_2_starts_when_timespan_1_stops(
                timespan, self)

    def stops_after_offset(self, offset):
        r'''Is true when timespan stops after `offset`. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)

        ::

            >>> timespan_1.starts_after_offset(Offset(-5))
            True
            >>> timespan_1.starts_after_offset(Offset(0))
            False
            >>> timespan_1.starts_after_offset(Offset(5))
            False

        Returns boolean.
        '''
        offset = durationtools.Offset(offset)
        return offset < self.stop_offset

    def stops_after_timespan_starts(self, timespan):
        r'''Is true when timespan stops when `timespan` starts. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(10, 20)

        ::

            >>> timespan_1.stops_after_timespan_starts(timespan_1)
            True
            >>> timespan_1.stops_after_timespan_starts(timespan_2)
            False
            >>> timespan_2.stops_after_timespan_starts(timespan_1)
            True
            >>> timespan_2.stops_after_timespan_starts(timespan_2)
            True

        Returns boolean.
        '''
        from abjad.tools import timespantools
        if self._implements_timespan_interface(timespan):
            return timespantools.timespan_2_stops_after_timespan_1_starts(
                timespan, self)

    def stops_after_timespan_stops(self, timespan):
        r'''Is true when timespan stops when `timespan` stops. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(10, 20)

        ::

            >>> timespan_1.stops_after_timespan_stops(timespan_1)
            False
            >>> timespan_1.stops_after_timespan_stops(timespan_2)
            False
            >>> timespan_2.stops_after_timespan_stops(timespan_1)
            True
            >>> timespan_2.stops_after_timespan_stops(timespan_2)
            False

        Returns boolean.
        '''
        from abjad.tools import timespantools
        if self._implements_timespan_interface(timespan):
            return timespantools.timespan_2_stops_after_timespan_1_stops(
                timespan, self)

    def stops_at_offset(self, offset):
        r'''Is true when timespan stops at `offset`. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)

        ::

            >>> timespan_1.stops_at_offset(Offset(-5))
            False
            >>> timespan_1.stops_at_offset(Offset(0))
            False
            >>> timespan_1.stops_at_offset(Offset(5))
            False

        Returns boolean.
        '''
        offset = durationtools.Offset(offset)
        return self.stop_offset == offset

    def stops_at_or_after_offset(self, offset):
        r'''Is true when timespan stops at or after `offset`. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)

        ::

            >>> timespan_1.stops_at_or_after_offset(Offset(-5))
            True
            >>> timespan_1.stops_at_or_after_offset(Offset(0))
            True
            >>> timespan_1.stops_at_or_after_offset(Offset(5))
            True

        Returns boolean.
        '''
        offset = durationtools.Offset(offset)
        return offset <= self.stop_offset

    def stops_before_offset(self, offset):
        r'''Is true when timespan stops before `offset`. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)

        ::

            >>> timespan_1.stops_before_offset(Offset(-5))
            False
            >>> timespan_1.stops_before_offset(Offset(0))
            False
            >>> timespan_1.stops_before_offset(Offset(5))
            False

        Returns boolean.
        '''
        offset = durationtools.Offset(offset)
        return self.stop_offset < offset

    def stops_before_or_at_offset(self, offset):
        r'''Is true when timespan stops before or at `offset`. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)

        ::

            >>> timespan_1.stops_before_or_at_offset(Offset(-5))
            False
            >>> timespan_1.stops_before_or_at_offset(Offset(0))
            False
            >>> timespan_1.stops_before_or_at_offset(Offset(5))
            False

        Returns boolean.
        '''
        offset = durationtools.Offset(offset)
        return self.stop_offset <= offset

    def stops_before_timespan_starts(self, timespan):
        r'''Is true when timespan stops before `timespan` starts.
        Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(10, 20)

        ::

            >>> timespan_1.stops_before_timespan_starts(timespan_1)
            False
            >>> timespan_1.stops_before_timespan_starts(timespan_2)
            False
            >>> timespan_2.stops_before_timespan_starts(timespan_1)
            False
            >>> timespan_2.stops_before_timespan_starts(timespan_2)
            False

        Returns boolean.
        '''
        from abjad.tools import timespantools
        if self._implements_timespan_interface(timespan):
            return timespantools.timespan_2_stops_before_timespan_1_starts(
                timespan, self)

    def stops_before_timespan_stops(self, timespan):
        r'''Is true when timespan stops before `timespan` stops. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(10, 20)

        ::

            >>> timespan_1.stops_before_timespan_stops(timespan_1)
            False
            >>> timespan_1.stops_before_timespan_stops(timespan_2)
            True
            >>> timespan_2.stops_before_timespan_stops(timespan_1)
            False
            >>> timespan_2.stops_before_timespan_stops(timespan_2)
            False

        Returns boolean.
        '''
        from abjad.tools import timespantools
        if self._implements_timespan_interface(timespan):
            return timespantools.timespan_2_stops_before_timespan_1_stops(
                timespan, self)

    def stops_during_timespan(self, timespan):
        r'''Is true when timespan stops during `timespan`. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(10, 20)

        ::

            >>> timespan_1.stops_during_timespan(timespan_1)
            True
            >>> timespan_1.stops_during_timespan(timespan_2)
            False
            >>> timespan_2.stops_during_timespan(timespan_1)
            False
            >>> timespan_2.stops_during_timespan(timespan_2)
            True

        Returns boolean.
        '''
        from abjad.tools import timespantools
        if self._implements_timespan_interface(timespan):
            return timespantools.timespan_2_stops_during_timespan_1(
                timespan, self)

    def stops_when_timespan_starts(self, timespan):
        r'''Is true when timespan stops when `timespan` starts. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(10, 20)

        ::

            >>> timespan_1.stops_when_timespan_starts(timespan_1)
            False
            >>> timespan_1.stops_when_timespan_starts(timespan_2)
            True
            >>> timespan_2.stops_when_timespan_starts(timespan_1)
            False
            >>> timespan_2.stops_when_timespan_starts(timespan_2)
            False

        Returns boolean.
        '''
        from abjad.tools import timespantools
        if self._implements_timespan_interface(timespan):
            return timespantools.timespan_2_stops_when_timespan_1_starts(
                timespan, self)

    def stops_when_timespan_stops(self, timespan):
        r'''Is true when timespan stops when `timespan` stops. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(10, 20)

        ::

            >>> timespan_1.stops_when_timespan_stops(timespan_1)
            True
            >>> timespan_1.stops_when_timespan_stops(timespan_2)
            False
            >>> timespan_2.stops_when_timespan_stops(timespan_1)
            False
            >>> timespan_2.stops_when_timespan_stops(timespan_2)
            True

        Returns boolean.
        '''
        from abjad.tools import timespantools
        if self._implements_timespan_interface(timespan):
            return timespantools.timespan_2_stops_when_timespan_1_stops(
                timespan, self)

    def stretch(self, multiplier, anchor=None):
        r'''Stretches timespan by `multiplier` relative to `anchor`.

        .. container:: example

            **Example 1.** Stretch relative to timespan start offset:

            ::

                >>> timespantools.Timespan(3, 10).stretch(Multiplier(2))
                Timespan(start_offset=Offset(3, 1), stop_offset=Offset(17, 1))

        .. container:: example

            **Example 2.** Stretch relative to timespan stop offset:

            ::

                >>> timespantools.Timespan(3, 10).stretch(Multiplier(2), Offset(10))
                Timespan(start_offset=Offset(-4, 1), stop_offset=Offset(10, 1))

        .. container:: example

            **Example 3.** Stretch relative to offset prior to timespan:

            ::

                >>> timespantools.Timespan(3, 10).stretch(Multiplier(2), Offset(0))
                Timespan(start_offset=Offset(6, 1), stop_offset=Offset(20, 1))

        .. container:: example

            **Example 4.** Stretch relative to offset after timespan:

            ::

                >>> timespantools.Timespan(3, 10).stretch(Multiplier(3), Offset(12))
                Timespan(start_offset=Offset(-15, 1), stop_offset=Offset(6, 1))

        .. container:: example

            **Example 5.** Stretch relative to offset that happens during timespan:

            ::

                >>> timespantools.Timespan(3, 10).stretch(Multiplier(2), Offset(4))
                Timespan(start_offset=Offset(2, 1), stop_offset=Offset(16, 1))

        Returns newly emitted timespan.
        '''
        from abjad.tools.topleveltools import new
        multiplier = durationtools.Multiplier(multiplier)
        assert 0 < multiplier
        if anchor is None:
            anchor = self.start_offset
        new_start_offset = (multiplier * (self.start_offset - anchor)) + anchor
        new_stop_offset =  (multiplier * (self.stop_offset - anchor)) + anchor
        result = new(
            self,
            start_offset=new_start_offset,
            stop_offset=new_stop_offset,
            )
        return result

    def translate(self, translation=None):
        r'''Translates timespan by `translation`.

            >>> timespan = timespantools.Timespan(5, 10)

        ::

            >>> timespan.translate(2)
            Timespan(start_offset=Offset(7, 1), stop_offset=Offset(12, 1))

        Returns new timespan.
        '''
        return self.translate_offsets(translation, translation)

    def translate_offsets(
        self,
        start_offset_translation=None,
        stop_offset_translation=None,
        ):
        r'''Translates timespan start offset by `start_offset_translation` and
        stop offset by `stop_offset_translation`:

        ::

            >>> timespan = timespantools.Timespan((1, 2), (3, 2))

        ::

            >>> timespan.translate_offsets(
            ...     start_offset_translation=Duration(-1, 8))
            Timespan(start_offset=Offset(3, 8), stop_offset=Offset(3, 2))

        Returns new timespan.
        '''
        start_offset_translation = start_offset_translation or 0
        stop_offset_translation = stop_offset_translation or 0
        start_offset_translation = \
            durationtools.Duration(start_offset_translation)
        stop_offset_translation = \
            durationtools.Duration(stop_offset_translation)
        new_start_offset = self.start_offset + start_offset_translation
        new_stop_offset = self.stop_offset + stop_offset_translation
        result = new(
            self,
            start_offset=new_start_offset,
            stop_offset=new_stop_offset,
            )
        return result

    def trisects_timespan(self, timespan):
        r'''Is true when timespan trisects `timespan`. Otherwise false:

        ::

            >>> timespan_1 = timespantools.Timespan(0, 10)
            >>> timespan_2 = timespantools.Timespan(5, 6)

        ::

            >>> timespan_1.trisects_timespan(timespan_1)
            False
            >>> timespan_1.trisects_timespan(timespan_2)
            False
            >>> timespan_2.trisects_timespan(timespan_1)
            True
            >>> timespan_2.trisects_timespan(timespan_2)
            False

        Returns boolean.
        '''
        from abjad.tools import timespantools
        if self._implements_timespan_interface(timespan):
            return timespantools.timespan_2_trisects_timespan_1(
                timespan, self)



# -*- encoding: utf-8 -*-


def is_positive_integer_equivalent_number(expr):
    '''Is true when `expr` is a positive integer-equivalent number.
    Otherwise false:

    ::

        >>> mathtools.is_positive_integer_equivalent_number(Duration(4, 2))
        True

    Returns boolean.
    '''
    from abjad.tools import mathtools

    try:
        return 0 < expr and mathtools.is_integer_equivalent_number(expr)
    except TypeError:  # Python 3 comparisons with non-numbers
        return False



# -*- encoding: utf-8 -*-
from abjad.tools import scoretools
from abjad.tools import sequencetools
from abjad.tools import spannertools
from abjad.tools.abctools import AbjadValueObject
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import detach
from abjad.tools.topleveltools import inspect_
from abjad.tools.topleveltools import iterate


class TieSpecifier(AbjadValueObject):
    r'''Tie specifier.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_tie_across_divisions',
        '_tie_split_notes',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        tie_across_divisions=False,
        tie_split_notes=True,
        ):
        assert isinstance(tie_across_divisions, bool)
        assert isinstance(tie_split_notes, bool)
        self._tie_across_divisions = tie_across_divisions
        self._tie_split_notes = tie_split_notes

    ### SPECIAL METHODS ###

    def __eq__(self, arg):
        r'''Is true when `arg` is a tie specifier with values of
        `tie_across_divisions` and `tie_split_notes` equal to those of this tie
        specifier. Otherwise false.

        Returns boolean.
        '''
        if isinstance(arg, type(self)):
            if self.tie_across_divisions == arg.tie_across_divisions and \
                self.tie_split_notes == arg.tie_split_notes:
                return True
        return False

    def __hash__(self):
        r'''Hashes tie specifier.

        Required to be explicitely re-defined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(TieSpecifier, self).__hash__()

    ### PRIVATE PROPERTIES ###

    @property
    def _attribute_manifest(self):
        from abjad.tools import systemtools
        from scoremanager import idetools
        return systemtools.AttributeManifest(
            systemtools.AttributeDetail(
                name='tie_across_divisions',
                command='tad',
                editor=idetools.getters.get_boolean,
                ),
            systemtools.AttributeDetail(
                name='tie_split_notes',
                command='tsn',
                editor=idetools.getters.get_boolean,
                ),
            )

    ### PRIVATE METHODS ###

    def _make_ties(self, music):
        if self.tie_across_divisions:
            self._make_ties_across_divisions(music)

    @staticmethod
    def _make_ties_across_divisions(music):
        for division_one, division_two in \
            sequencetools.iterate_sequence_nwise(music):
            leaf_one = next(iterate(division_one).by_class(
                prototype=scoretools.Leaf,
                reverse=True))
            leaf_two = next(iterate(division_two).by_class(
                prototype=scoretools.Leaf))
            leaves = [leaf_one, leaf_two]
            prototype = (scoretools.Note, scoretools.Chord)
            if not all(isinstance(x, prototype) for x in leaves):
                continue
            logical_tie_one = inspect_(leaf_one).get_logical_tie()
            logical_tie_two = inspect_(leaf_two).get_logical_tie()
            for tie in inspect_(leaf_one).get_spanners(spannertools.Tie):
                detach(tie, leaf_one)
            for tie in inspect_(leaf_two).get_spanners(spannertools.Tie):
                detach(tie, leaf_two)
            combined_logical_tie = logical_tie_one + logical_tie_two
            attach(spannertools.Tie(), combined_logical_tie)

    ### PUBLIC PROPERTIES ###

    @property
    def tie_across_divisions(self):
        r'''Is true when rhythm-maker should tie across divisons.
        Otherwise false.

        Returns boolean.
        '''
        return self._tie_across_divisions

    @property
    def tie_split_notes(self):
        r'''Is true when rhythm-maker should tie split notes.
        Otherwise false.

        Returns boolean.
        '''
        return self._tie_split_notes


# -*- encoding: utf-8 -*-
import inspect
import os
from abjad.tools import documentationtools


def get_developer_script_classes():
    r'''Returns a list of all developer script classes.
    '''
    from abjad.tools import developerscripttools
    from abjad.tools import abjadbooktools

    tools_package_paths = [
        abjadbooktools.__path__[0],
        developerscripttools.__path__[0],
    ]
    script_classes = []
    for tools_package_path in tools_package_paths:
        developer_script_classes = documentationtools.ClassCrawler(
            tools_package_path, root_package_name='abjad')()
        for developer_script_class in developer_script_classes:
            if developerscripttools.DeveloperScript in \
                inspect.getmro(developer_script_class) and \
                not inspect.isabstract(developer_script_class):
                script_classes.append(developer_script_class)

    return list(sorted(script_classes, key=lambda x: x.__name__))


# -*- encoding: utf-8 -*-
from abjad import *


def test_datastructuretools_CyclicTree___iter___01():
    r'''Empty cyclic tree iterates no elements.
    '''

    cyclic_tree = datastructuretools.CyclicPayloadTree([])

    for element in cyclic_tree:
        assert False


def test_datastructuretools_CyclicTree___iter___02():
    r'''Cyclic tree iterates top-level elements only once.
    '''

    cyclic_tree = datastructuretools.CyclicPayloadTree([[1, 2], [3, 4]])
    elements = list(iter(cyclic_tree))

    assert elements[0] is cyclic_tree[0]
    assert elements[1] is cyclic_tree[1]


# -*- encoding: utf-8 -*-
from abjad import *


def test_spannertools_Crescendo___init___01():
    r'''Initialize empty crescendo spanner.
    '''

    crescendo = Crescendo()
    assert isinstance(crescendo, Crescendo)


def test_spannertools_Crescendo___init___02():

    staff = Staff("c'8 d'8 e'8 f'8 g'2")
    crescendo = Crescendo()
    attach(crescendo, staff[:4])

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            c'8 \<
            d'8
            e'8
            f'8 \!
            g'2
        }
        '''
        )


# -*- encoding: utf-8 -*-


def all_are_nonnegative_integer_equivalent_numbers(expr):
    '''True `expr` is a sequence and when all elements in `expr` are
    nonnegative integer-equivalent numbers. Otherwise false:

    ::


        >>> expr = [0, 0.0, Fraction(0), 2, 2.0, Fraction(2)]
        >>> mathtools.all_are_nonnegative_integer_equivalent_numbers(expr)
        True

    Returns boolean.
    '''
    from abjad.tools import mathtools

    try:
        return all(
            mathtools.is_nonnegative_integer_equivalent_number(x)
            for x in expr
            )
    except TypeError:
        return False


# -*- encoding: utf-8 -*-
from abjad.tools.datastructuretools.TypedList import TypedList


class PerformerInventory(TypedList):
    r'''Abjad model of an ordered list of performers.

    Performer inventories implement the list interface and are mutable.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### PRIVATE PROPERTIES ###

    @property
    def _attribute_manifest(self):
        from abjad.tools import systemtools
        return systemtools.AttributeManifest()

    @property
    def _item_creator_class(self):
        item_creator_class = self._make_item_creator_class()
        return item_creator_class

    @property
    def _item_creator_class_kwargs(self):
        return {'is_ranged': True}

    ### PRIVATE METHODS ###

    @staticmethod
    def _initialize_performer(performer, session):
        from abjad.tools import instrumenttools
        menu = PerformerInventory._make_performer_configuration_menu(
            performer,
            session,
            )
        while True:
            result = menu._run()
            if menu._session.is_backtracking:
                return
            elif not result:
                continue
            if result in ('skip', ['skip']):
                break
            elif result in ('more', ['more']):
                inventory = instrumenttools.InstrumentInventory()
                class_ = inventory._make_item_creator_class()
                item_creator = class_(
                    session=session,
                    is_ranged=True,
                    )
                instruments = item_creator._run()
                if item_creator._session.is_backtracking:
                    break
                if instruments is not None:
                    for instrument in instruments:
                        performer.instruments.append(instrument)
                break
            elif isinstance(result, list):
                name = '_default_instrument_name_to_instrument_class'
                method = getattr(instrumenttools.Instrument, name)
                for instrument_name in result:
                    instrument_class = method(instrument_name)
                    instrument = instrument_class()
                    performer.instruments.append(instrument)
                break
            else:
                raise Exception("how'd we get here?")

    @staticmethod
    def _make_performer_configuration_menu(performer, session):
        from abjad.tools import instrumenttools
        menu = session._io_manager._make_menu(name='performer configuration')
        commands = []
        likely_instruments = \
            performer.likely_instruments_based_on_performer_name
        likely_instrument_names = [
            x().instrument_name for x in likely_instruments]
        likely_instrument_names.sort()
        most_likely_instrument = \
            performer.most_likely_instrument_based_on_performer_name
        default_index = None
        numbered_menu_entries = []
        if most_likely_instrument is not None:
            most_likely_instrument_name = \
                most_likely_instrument().instrument_name
            assert most_likely_instrument_name in likely_instrument_names
            most_likely_index = likely_instrument_names.index(
                most_likely_instrument_name)
            string = '{} (default)'.format(most_likely_instrument_name)
            likely_instrument_names[most_likely_index] = string
            most_likely_number = most_likely_index + 1
            default_index = most_likely_index
        if likely_instruments:
            numbered_menu_entries = likely_instrument_names
            commands.append(('instruments - more', 'more'))
        else:
            instrument_names = \
                instrumenttools.Instrument._list_instrument_names()
            numbered_menu_entries = instrument_names
        numbered_list_section = menu.make_numbered_list_section(
            default_index=default_index,
            group_by_annotation=False,
            menu_entries=numbered_menu_entries,
            name='select instruments',
            title='select instruments',
            )
        commands.append(('instruments - skip', 'skip'))
        menu.make_command_section(
            commands=commands,
            name='instrument commands',
            )
        return menu

    ### ITEM CREATOR ###

    @staticmethod
    def _make_item_creator_class():
        from scoremanager.idetools.Controller import Controller
        class ItemCreator(Controller):
            ### CLASS VARIABLES ###
            __slots__ = ('_is_ranged', '_target')
            ### INITIALIZER ###
            def __init__(self, is_ranged=False, session=None, target=None):
                Controller.__init__(self, session=session)
                self._is_ranged = is_ranged
                self._target = target
            ### PRIVATE METHODS ###
            def _run(self):
                from abjad.tools import instrumenttools
                from scoremanager import idetools
                try_again = False
                performers = []
                controller = idetools.ControllerContext(controller=self)
                selector = idetools.Selector(session=self._session)
                selector = selector.make_score_tools_performer_name_selector(
                    is_ranged=self._is_ranged,
                    )
                with controller:
                    while True:
                        result = selector._run()
                        if self._session.is_backtracking:
                            break
                        if isinstance(result, list):
                            performer_names = result
                        else:
                            performer_names = [result]
                        performers = []
                        Performer = instrumenttools.Performer
                        for performer_name in performer_names:
                            performer = Performer(performer_name)
                            PerformerInventory._initialize_performer(
                                performer,
                                self._session,
                                )
                            was_backtracking_locally = \
                                self._session.is_backtracking_locally
                            if self._session.is_backtracking:
                                if was_backtracking_locally:
                                    try_again = True
                                else:
                                    try_again = False
                                    performers = []
                                break
                            performers.append(performer)
                        if not try_again:
                            break
                        else:
                            try_again = False
                    if self._is_ranged and performers:
                        final_result = performers[:]
                    elif self._is_ranged and not performers:
                        final_result = []
                    elif not self._is_ranged and performers:
                        final_result = performers[0]
                    elif not self._is_ranged and not performers:
                        final_result = None
                    else:
                        raise ValueError
                    self._target = final_result
                    return self.target
            @property
            def target(self):
                return self._target
        return ItemCreator

    ### PUBLIC METHODS ###

    def get_instrument(self, instrument_name):
        r'''Gets first instrument in performer inventory with
        `instrument_name`.

        ..  container:: example

            ::
            
                >>> flutist = instrumenttools.Performer(name='flutist')
                >>> flutist.instruments.append(instrumenttools.Flute())
                >>> flutist.instruments.append(instrumenttools.Piccolo())
                >>> inventory = instrumenttools.PerformerInventory(
                ...     [flutist],
                ...     )
                >>> inventory.get_instrument('piccolo')
                Piccolo()

        Returns instrument or none.
        '''
        for performer in self:
            instrument = performer.get_instrument(instrument_name)
            if instrument is not None:
                return instrument


# -*- encoding: utf-8 -*-
import copy


def rotate_sequence(sequence, n):
    '''Rotates `sequence`.

    Rotates `sequence` to the right:

    ::

        >>> sequencetools.rotate_sequence(list(range(10)), 4)
        [6, 7, 8, 9, 0, 1, 2, 3, 4, 5]

    Rotates `sequence` to the left:

    ::

        >>> sequencetools.rotate_sequence(list(range(10)), -3)
        [3, 4, 5, 6, 7, 8, 9, 0, 1, 2]

    Rotates `sequence` neither to the right nor the left:

    ::

        >>> sequencetools.rotate_sequence(list(range(10)), 0)
        [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]

    Returns newly created `sequence` object.
    '''
    from abjad.tools import sequencetools
    result = sequencetools.Sequence(*sequence)
    result = result.rotate(n)
    return type(sequence)(result)


# -*- encoding: utf-8 -*-
import re
from abjad.tools.stringtools.is_snake_case \
	import underscore_delimited_lowercase_regex_body


underscore_delimited_lowercase_file_name_with_extension_regex_body = """
    %s
    \.
    [a-z,0-9]+
    """ % underscore_delimited_lowercase_regex_body

underscore_delimited_lowercase_file_name_with_extension_regex = re.compile(
    '^%s$' % 
    underscore_delimited_lowercase_file_name_with_extension_regex_body, 
    re.VERBOSE
    )

def is_snake_case_file_name_with_extension(expr):
    r'''Is true when `expr` is a string and is underscore-delimited lowercase 
    file name with extension.

    ..  container:: example

        ::

            >>> stringtools.is_snake_case_file_name_with_extension('foo_bar.blah')
            True

    Otherwise false:

    ..  container:: example

        ::

            >>> stringtools.is_snake_case_file_name_with_extension('foo.bar.blah')
            False

    Returns boolean.
    '''

    if not isinstance(expr, str):
        return False

    if expr == '':
        return True

    return bool(underscore_delimited_lowercase_file_name_with_extension_regex.match(expr))


# -*- encoding: utf-8 -*-
from abjad.tools.abctools import AbjadObject


class StatalServerCursor(AbjadObject):
    r'''A statal server cursor.
    '''

    ### INITIALIZER ###

    def __init__(
        self,
        statal_server=None,
        position=None,
        reverse=False,
        ):
        from abjad.tools import datastructuretools
        assert isinstance(position, (tuple, type(None))), repr(position)
        assert isinstance(reverse, type(True)), repr(reverse)
        position = position or ()
        self._statal_server = statal_server
        self._position = position
        self._reverse = reverse

    ### SPECIAL METHODS ###

    def __call__(self, n=1, level=-1):
        r'''Gets manifest payload of next `n` nodes at `level`.

        ..  container:: example

            Gets manifest payload of nodes at level -1:

            ::

                >>> sequence = [(0, 1), (2, 3), (4, 5), (6, 7)]
                >>> server = datastructuretools.StatalServer(sequence)
                >>> cursor = server()

            ::

                >>> cursor()
                [0]
                >>> cursor()
                [1]
                >>> cursor()
                [2]
                >>> cursor()
                [3]
                >>> cursor()
                [4]
                >>> cursor()
                [5]
                >>> cursor()
                [6]
                >>> cursor()
                [7]
                >>> cursor()
                [0]
                >>> cursor()
                [1]

        ..  container:: example

            Gets manifest payload of nodes at level -2:

            ::

                >>> sequence = [(0, 1), (2, 3), (4, 5), (6, 7)]
                >>> server = datastructuretools.StatalServer(sequence)
                >>> cursor = server()

            ::

                >>> cursor(level=-2)
                [0, 1]
                >>> cursor(level=-2)
                [2, 3]
                >>> cursor(level=-2)
                [4, 5]
                >>> cursor(level=-2)
                [6, 7]
                >>> cursor(level=-2)
                [0, 1]

        ..  container:: example

            Gets manifest payload of nodes at level -1 for statal server of
            length 1:

            ::

                >>> sequence = [0]
                >>> server = datastructuretools.StatalServer(sequence)
                >>> cursor = server()

            ::

                >>> cursor()
                [0]
                >>> cursor()
                [0]
                >>> cursor()
                [0]
                >>> cursor()
                [0]
                >>> cursor()
                [0]

        Returns list of arbitrary values.
        '''
        return self._get_manifest_payload_of_next_n_nodes_at_level(
            n, level=level)

    def __eq__(self, expr):
        r'''True `expr` is a statal server cursor and keyword
        argument values are equal. Otherwise false.

        Returns boolean.
        '''
        from abjad.tools import systemtools
        return systemtools.StorageFormatManager.compare(self, expr)

    def __hash__(self):
        r'''Hashes statal server cursor.

        Required to be explicitely re-defined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(StatalServerCursor, self).__hash__()

    ### PRIVATE METHODS ###

    def _get_manifest_payload_of_next_n_nodes_at_level(self, n=1, level=-1):
        result = []
        #print
        #print repr(self.position), '(position)'
        if not self.statal_server.cyclic_tree._is_valid_level(level):
            message = 'invalid level: {!r}.'.format(level)
            raise Exception(message)
        current_node = self.statal_server.cyclic_tree.get_node_at_position(
            self.position)
        if self.reverse:
            n *= -1
        nodes = current_node.get_next_n_nodes_at_level(n, level)
        position = nodes[-1].position
        self._position = position
        for node in nodes:
            result.extend(node.manifest_payload)
        #print repr(result), '(result)'
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def position(self):
        r'''Statal server cursor position.

        Returns tuple.
        '''
        return self._position

    @property
    def reverse(self):
        r'''Statal server cursor reverse.

        False when cursor reads from left to right.
        Is true when cursor reads from right to left.

        Returns boolean.
        '''
        return self._reverse

    @property
    def statal_server(self):
        r'''Statal server cursor statal server.

        Returns statal server.
        '''
        return self._statal_server


# -*- encoding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools import scoretools
from abjad.tools.rhythmmakertools.RhythmMaker import RhythmMaker


class SkipRhythmMaker(RhythmMaker):
    r'''Skip rhythm-maker.

    ..  container:: example

        Makes skips equal to the duration of input divisions.

        ::

            >>> maker = rhythmmakertools.SkipRhythmMaker()

        ::

            >>> divisions = [(1, 4), (3, 16), (5, 8)]
            >>> music = maker(divisions)
            >>> lilypond_file = rhythmmakertools.make_lilypond_file(
            ...     music,
            ...     divisions,
            ...     )
            >>> show(lilypond_file) # doctest: +SKIP

        ..  doctest::

            >>> staff = maker._get_rhythmic_staff(lilypond_file)
            >>> f(staff)
            \new RhythmicStaff {
                {
                    \time 1/4
                    s1 * 1/4
                }
                {
                    \time 3/16
                    s1 * 3/16
                }
                {
                    \time 5/8
                    s1 * 5/8
                }
            }

    Usage follows the two-step configure-once / call-repeatedly pattern shown
    here.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    _class_name_abbreviation = 'S'

    _human_readable_class_name = 'skip rhythm-maker'

    ### INITIALIZER ###

    def __init__(
        self,
        ):
        RhythmMaker.__init__(
            self,
            )

    ### SPECIAL METHODS ###

    def __call__(self, divisions, seeds=None):
        r'''Calls skip rhythm-maker on `divisions`.

        Returns list of selections.
        '''
        return RhythmMaker.__call__(
            self,
            divisions,
            seeds=seeds,
            )

    def __format__(self, format_specification=''):
        r'''Formats skip rhythm-maker.

        Set `format_specification` to `''` or `'storage'`.

        ..  container:: example

            ::

                >>> print(format(maker))
                rhythmmakertools.SkipRhythmMaker()

        Returns string.
        '''
        superclass = super(SkipRhythmMaker, self)
        return superclass.__format__(format_specification=format_specification)

    ### PRIVATE METHODS ###

    def _make_music(self, duration_pairs, seeds):
        result = []
        for duration_pair in duration_pairs:
            written_duration = durationtools.Duration(1)
            multiplied_duration = duration_pair
            skip = scoretools.make_skips_with_multiplied_durations(
                written_duration, [multiplied_duration])
            result.append(skip)
        return result

    ### PUBLIC METHODS ###

    def reverse(self):
        r'''Reverses skip rhythm-maker.

        ..  container:: example

            ::

                >>> reversed_maker = maker.reverse()

            ::

                >>> print(format(reversed_maker))
                rhythmmakertools.SkipRhythmMaker()

            ::

                >>> divisions = [(1, 4), (3, 16), (5, 8)]
                >>> music = maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     music,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> staff = maker._get_rhythmic_staff(lilypond_file)
                >>> f(staff)
                \new RhythmicStaff {
                    {
                        \time 1/4
                        s1 * 1/4
                    }
                    {
                        \time 3/16
                        s1 * 3/16
                    }
                    {
                        \time 5/8
                        s1 * 5/8
                    }
                }

        Defined equal to copy of rhythm-maker.

        Returns new skip rhythm-maker.
        '''
        return type(self)()


# -*- encoding: utf-8 -*-
from abjad import *


def test_mathtools_all_are_integer_equivalent_numbers_01():

    numbers = [1, 2, 3.0, Fraction(4)]
    assert mathtools.all_are_integer_equivalent_numbers(numbers)


def test_mathtools_all_are_integer_equivalent_numbers_02():

    numbers = [1, 2, 3.5, 4]
    assert not mathtools.all_are_integer_equivalent_numbers(numbers)

    numbers = [1, 2, 3, Fraction(4, 11)]
    assert not mathtools.all_are_integer_equivalent_numbers(numbers)

    assert not mathtools.all_are_integer_equivalent_numbers(7)

    assert not mathtools.all_are_integer_equivalent_numbers('foo')


# -*- encoding: utf-8 -*-
from abjad.tools import rhythmtreetools
import copy


def test_rhythmtreetools_RhythmTreeContainer___copy___01():

    string = '(1 (1 (2 (3 (4 (1 1 1)))) 2))'
    tree = rhythmtreetools.RhythmTreeParser()(string)[0]
    copied = copy.copy(tree)

    assert tree == copied
    assert tree is not copied

    assert tree[0] == copied[0]
    assert tree[0] is not copied[0]

    assert tree[1] == copied[1]
    assert tree[1] is not copied[1]

    assert tree[2] == copied[2]
    assert tree[2] is not copied[2]


# -*- encoding: utf-8 -*-
import itertools


def partition_sequence_by_value_of_elements(sequence):
    '''Groups `sequence` elements by value of elements.

    ::

        >>> sequence = [0, 0, -1, -1, 2, 3, -5, 1, 1, 5, -5]

    ::

        >>> sequencetools.partition_sequence_by_value_of_elements(sequence)
        [(0, 0), (-1, -1), (2,), (3,), (-5,), (1, 1), (5,), (-5,)]

    Returns list of tuples of `sequence` element references.
    '''
    result = []
    g = itertools.groupby(sequence, lambda x: x)
    for n, group in g:
        result.append(tuple(group))
    return result


# -*- encoding: utf-8 -*-
import os
import subprocess
import sys
from abjad.tools.systemtools.Configuration import Configuration


class AbjadConfiguration(Configuration):
    r'''Abjad configuration.

    ..  container:: example

        ::

            >>> abjad_configuration = systemtools.AbjadConfiguration()

        ::

            >>> abjad_configuration['accidental_spelling']
            'mixed'

    `AbjadConfiguration` creates the `$HOME/.abjad/` directory
    on instantiation.

    `AbjadConfiguration` then attempts to read an `abjad.cfg` file in
    that directory and parse the file as a `ConfigObj` configuration.

    `AbjadConfiguration` generates a default configuration if no file
    is found.

    `AbjadConfiguration` validates the `ConfigObj` instance
    and replaces key-value pairs which fail validation with default values.

    `AbjadConfiguration` then writes the configuration back to disk.

    The Abjad output directory is created the from
    `abjad_output_directory` key if it does not already exist.

    `AbjadConfiguration` supports the mutable mapping interface
    and can be subscripted as a dictionary.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### INITIALIZER ###

    def __init__(self):
        Configuration.__init__(self)
        if not os.path.exists(self.abjad_output_directory):
            os.makedirs(self.abjad_output_directory)

    ### PRIVATE METHODS ###

    def _get_option_definitions(self):
        options = {
            'abjad_output_directory': {
                'comment': [
                    '',
                    'Set to the directory where all Abjad-generated files',
                    '(such as PDFs and LilyPond files) should be saved.',
                    'Defaults to $HOME/.abjad/output/'
                ],
                'spec': 'string(default={!r})'.format(
                    os.path.join(
                        self.abjad_configuration_directory,
                        'output',
                        )
                    )
            },
            'accidental_spelling': {
                'comment': [
                    '',
                    'Default accidental spelling (mixed|sharps|flats).',
                ],
                'spec': "option('mixed', 'sharps', 'flats', default='mixed')"
            },
            'lilypond_includes': {
                'comment': [
                    '',
                    'Comma-separated list of LilyPond files that ',
                    'Abjad will "\include" in all generated *.ly files',
                ],
                'spec': 'list(min=0, default=list())'
            },
            'lilypond_language': {
                'comment': [
                    '',
                    'Language to use in all generated LilyPond files.'
                ],
                'spec': "string(default='english')"
            },
            'lilypond_path': {
                'comment': [
                    '',
                    'Lilypond executable path. Set to override dynamic lookup.'
                ],
                'spec': "string(default='lilypond')"
            },
            'midi_player': {
                'comment': [
                    '',
                    'MIDI player to open MIDI files.',
                    'When unset your OS should know how to open MIDI files.',
                ],
                'spec': "string(default='')"
            },
            'pdf_viewer': {
                'comment': [
                    '',
                    'PDF viewer to open PDF files.',
                    'When unset your OS should know how to open PDFs.',
                ],
                'spec': "string(default='')"
            },
            'text_editor': {
                'comment': [
                    '',
                    'Text editor to edit text files.',
                    'When unset your OS should know how to open text files.'
                ],
                'spec': "string(default='')"
            },

        }
        return options


    ### PRIVATE PROPERTIES ###

    @property
    def _initial_comment(self):
        current_time = self._current_time
        return [
            '-*- coding: utf-8 -*-',
            '',
            'Abjad configuration file created on {}.'.format(current_time),
            'This file is interpreted by ConfigObj and follows ini sytnax.',
        ]

    ### PUBLIC METHODS ###

    @classmethod
    def get_abjad_startup_string(cls):
        r'''Gets Abjad startup string.

        ..  container:: example

            ::

                >>> abjad_configuration.get_abjad_startup_string()
                'Abjad 2.15 (development)'

        Returns string.
        '''
        result = 'Abjad {} ({})'
        result = result.format(
            cls.get_abjad_version_string(),
            'development',
            )
        return result

    @staticmethod
    def get_abjad_version_string():
        '''Gets Abjad version string.

        ..  container:: example

            ::

                >>> abjad_configuration.get_abjad_version_string()
                '2.15'

        Returns string.
        '''
        import abjad
        return abjad.__version__

    @classmethod
    def get_lilypond_minimum_version_string(cls):
        r'''Gets LilyPond minimum version string.

        ..  container:: example

            ::

                >>> abjad_configuration.get_lilypond_minimum_version_string() # doctest: +SKIP
                '2.17.0'

        Returns string.
        '''
        version = cls.get_lilypond_version_string()
        parts = version.split('.')[0:2]
        parts.append('0')
        return '.'.join(parts)

    @staticmethod
    def get_lilypond_version_string():
        '''Gets LilyPond version string.

        ..  container:: example

            ::

                >>> abjad_configuration.get_lilypond_version_string() # doctest: +SKIP
                '2.19.1'

        Returns string.
        '''
        from abjad import abjad_configuration
        if subprocess.mswindows and not 'LilyPond' in os.environ.get('PATH'):
            command = r'dir "C:\Program Files\*.exe" /s /b | find "lilypond.exe"'
            proc = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE)
            lilypond = proc.stdout.readline()
            lilypond = lilypond.strip('\r').strip('\n').strip()
            if lilypond == '':
                message = 'can not find LilyPond under Windows.'
                raise SystemError(message)
        else:
            lilypond = abjad_configuration['lilypond_path']
            if not lilypond:
                lilypond = 'lilypond'
        command = lilypond + ' --version'
        proc = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE)
        if sys.version_info[0] == 2:
            lilypond_version_string = proc.stdout.readline()
        else:
            import locale
            encoding = locale.getdefaultlocale()[1]
            lilypond_version_string = proc.stdout.readline().decode(encoding)
        lilypond_version_string = lilypond_version_string.split(' ')[-1]
        lilypond_version_string = lilypond_version_string.strip()
        return lilypond_version_string

    @staticmethod
    def get_python_version_string():
        '''Gets Python version string.

        ..  container:: example

            ::

                >>> abjad_configuration.get_python_version_string() # doctest: +SKIP
                '2.7.5'

        Returns string.
        '''
        # python prints to stderr on startup (instead of stdout)
        command = 'python --version'
        proc = subprocess.Popen(command, shell=True, stderr=subprocess.PIPE)
        # massage output string
        if sys.version_info[0] == 2:
            python_version_string = proc.stderr.readline()
        else:
            import locale
            encoding = locale.getdefaultlocale()[1]
            python_version_string = proc.stderr.readline().decode(encoding)
        python_version_string = python_version_string.split(' ')[-1]
        python_version_string = python_version_string.strip()
        # return trimmed string
        return python_version_string

    @staticmethod
    def get_tab_width():
        r'''Gets tab width.

        ..  container:: example

            ::

                >>> abjad_configuration.get_tab_width()
                4

        Used by code generation functions.

        Returns nonnegative integer.
        '''
        return 4

    @staticmethod
    def get_text_editor():
        r'''Gets text editor.

        ..  container:: example

            ::

                >>> abjad_configuration.get_text_editor() # doctest: +SKIP
                'vim'

        Returns string.
        '''
        from abjad import abjad_configuration
        text_editor = abjad_configuration['text_editor']
        if text_editor is not None:
            return text_editor
        elif os.name == 'posix':
            return 'vim'
        else:
            return 'edit'

    @staticmethod
    def list_package_dependency_versions():
        r'''Lists package dependency versions.

        ..  container:: example

            ::

                >>> abjad_configuration.list_package_dependency_versions() # doctest: +SKIP
                {'sphinx': '1.1.2', 'pytest': '2.1.2'}

        Returns dictionary.
        '''
        dependencies = {}
        dependencies['configobj'] = None
        try:
            import configobj
            dependencies['configobj'] = configobj.__version__
        except (AttributeError, ImportError):
            pass
        dependencies['ply'] = None
        try:
            from ply import lex
            dependencies['ply'] = lex.__version__
        except ImportError:
            pass
        dependencies['pytest'] = None
        try:
            import pytest
            dependencies['pytest'] = pytest.__version__
        except ImportError:
            pass
        dependencies['sphinx'] = None
        try:
            import sphinx
            dependencies['sphinx'] = sphinx.__version__
        except ImportError:
            pass
        return dependencies

    @staticmethod
    def set_default_accidental_spelling(spelling='mixed'):
        '''Sets default accidental spelling.

        ..  container:: example

            Sets default accidental spelling to sharps:

            ::

                >>> abjad_configuration.set_default_accidental_spelling('sharps')

            ::

                >>> [Note(13, (1, 4)), Note(15, (1, 4))]
                [Note("cs''4"), Note("ds''4")]

        ..  container:: example

            Sets default accidental spelling to flats:

            ::

                >>> abjad_configuration.set_default_accidental_spelling('flats')

            ::

                >>> [Note(13, (1, 4)), Note(15, (1, 4))]
                [Note("df''4"), Note("ef''4")]

        ..  container:: example

            Sets default accidental spelling to mixed:

            ::

                >>> abjad_configuration.set_default_accidental_spelling()

            ::

                >>> [Note(13, (1, 4)), Note(15, (1, 4))]
                [Note("cs''4"), Note("ef''4")]

        Defaults to ``'mixed'``.

        Mixed test case must appear last here for doc tests to check correctly.

        Returns none.
        '''
        from abjad import abjad_configuration
        if spelling not in ('mixed', 'sharps', 'flats'):
            raise ValueError
        abjad_configuration['accidental_spelling'] = spelling

    ### PUBLIC PROPERTIES ###

    @property
    def abjad_configuration_directory(self):
        r'''Abjad configuration directory.

        Returns string.
        '''
        relative_path = os.path.join(
            self.home_directory,
            '.abjad',
            )
        return os.path.abspath(relative_path)

    @property
    def abjad_configuration_file_path(self):
        r'''Abjad configuration file path.

        Returns string.
        '''
        return self.configuration_file_path

    @property
    def abjad_directory(self):
        r'''Abjad directory.

        Returns string.
        '''
        module_parts = self.__module__.split('.')
        file_path_parts = os.path.abspath(__file__).rpartition('.py')
        file_path_parts = file_path_parts[0].split(os.path.sep)
        for part in reversed(module_parts):
            if part == 'abjad':
                break
            file_path_parts.pop()
        return os.path.sep.join(file_path_parts)

    @property
    def abjad_experimental_directory(self):
        r'''Abjad experimental directory.

        Returns string.
        '''
        relative_path = os.path.join(
            self.abjad_root_directory,
            'experimental',
            )
        return os.path.abspath(relative_path)

    @property
    def abjad_output_directory(self):
        r'''Abjad output directory.

        Returns string.
        '''
        if 'abjad_output_directory' in self._settings:
            return self._settings['abjad_output_directory']
        return os.path.join(
            self.abjad_configuration_directory,
            'output'
            )

    @property
    def abjad_root_directory(self):
        r'''Abjad root directory.

        Returns string.
        '''
        relative_path = os.path.join(
            self.abjad_directory,
            '..',
            )
        return os.path.abspath(relative_path)

    @property
    def abjad_stylesheets_directory(self):
        r'''Abjad stylesheets directory.

        Returns string.
        '''
        relative_path = os.path.join(
            self.abjad_directory,
            'stylesheets',
            )
        return os.path.abspath(relative_path)

    @property
    def abjad_tools_directory(self):
        r'''Abjad tools directory.

        Returns string.
        '''
        relative_path = os.path.join(
            self.abjad_directory,
            'tools',
            )
        return os.path.abspath(relative_path)

    @property
    def configuration_directory(self):
        r'''Configuration directory.

        Returns string.
        '''
        return self.abjad_configuration_directory

    @property
    def configuration_file_name(self):
        r'''Configuration file name.

        Returns string.
        '''
        return 'abjad.cfg'

    @property
    def score_manager_directory(self):
        r'''Abjad score manager directory.

        Returns string.
        '''
        relative_path = os.path.join(
            self.abjad_root_directory,
            'scoremanager'
            )
        return os.path.abspath(relative_path)


# -*- encoding: utf-8 -*-
import pytest
from abjad import *


def test_mathtools_cumulative_sums_01():
    r'''Returns list of the cumulative sums of the integer elements in input.
    '''

    assert mathtools.cumulative_sums([1, 2, 3], start=None) == [1, 3, 6]
    assert mathtools.cumulative_sums([10, -9, -8], start=None) == [10, 1, -7]
    assert mathtools.cumulative_sums([0, 0, 0, 5], start=None) == [0, 0, 0, 5]
    assert mathtools.cumulative_sums([-10, 10, -10, 10], start=None) == \
        [-10, 0, -10, 0]


def test_mathtools_cumulative_sums_02():
    r'''Raise exception when sequence is neither tuple nor list.
    '''

    assert pytest.raises(Exception, "mathtools.cumulative_sums('foo')")


# -*- encoding: utf-8 -*-


def make_vertically_adjusted_composer_markup(
    composer,
    font_name='Times',
    font_size=3,
    space_above=20,
    space_right=0,
    ):
    r'''Makes vertically adjusted `composer` markup.

    ::

        >>> markup = markuptools.make_vertically_adjusted_composer_markup('Josquin Desprez')

    ::

        >>> print(format(markup, 'lilypond'))
        \markup {
            \override
                #'(font-name . "Times")
                {
                    \hspace
                        #0
                    \raise
                        #-20
                        \fontsize
                            #3
                            "Josquin Desprez"
                    \hspace
                        #0
                }
            }

    Returns markup.
    '''
    from abjad.tools import markuptools

    assert isinstance(composer, str)
    assert isinstance(font_name, str)
    assert isinstance(font_size, (int, float))
    assert isinstance(space_above, (int, float))
    assert isinstance(space_right, (int, float))

    contents = r'''
        \override #'(font-name . "%s") {
            \hspace #0
            \raise #-%s \fontsize #%s "%s"
            \hspace #%s
        }
       ''' % (font_name, space_above, font_size, composer, space_right)

    return markuptools.Markup(contents)



# -*- encoding: utf-8 -*-
import os
from abjad.tools.datastructuretools.PayloadTree import PayloadTree


class PitchClassTree(PayloadTree):
    r'''A pitch-class tree.

    ..  container:: example

        Numbered pitch-class tree::

            >>> tree = pitchtools.PitchClassTree(
            ...     items=[[0, 4, 7, 8], [9, 2, 3, 11]],
            ...     item_class=pitchtools.NumberedPitchClass,
            ...     )
            >>> print(format(tree, 'storage'))
            pitchtools.PitchClassTree(
                [
                    [
                        pitchtools.NumberedPitchClass(0),
                        pitchtools.NumberedPitchClass(4),
                        pitchtools.NumberedPitchClass(7),
                        pitchtools.NumberedPitchClass(8),
                        ],
                    [
                        pitchtools.NumberedPitchClass(9),
                        pitchtools.NumberedPitchClass(2),
                        pitchtools.NumberedPitchClass(3),
                        pitchtools.NumberedPitchClass(11),
                        ],
                    ]
                )

    ..  container:: example

        Named pitch-class tree::

            >>> tree = pitchtools.PitchClassTree(
            ...     items=[['c', 'e', 'g', 'af'], ['a', 'd', 'ef', 'b']],
            ...     item_class=pitchtools.NamedPitchClass,
            ...     )
            >>> print(format(tree, 'storage'))
            pitchtools.PitchClassTree(
                [
                    [
                        pitchtools.NamedPitchClass('c'),
                        pitchtools.NamedPitchClass('e'),
                        pitchtools.NamedPitchClass('g'),
                        pitchtools.NamedPitchClass('af'),
                        ],
                    [
                        pitchtools.NamedPitchClass('a'),
                        pitchtools.NamedPitchClass('d'),
                        pitchtools.NamedPitchClass('ef'),
                        pitchtools.NamedPitchClass('b'),
                        ],
                    ]
                )

    Pitch-class trees are treated as immutable.
    '''

    ### INITIALIZER ###

    def __init__(
        self,
        items=None,
        item_class=None,
        ):
        PayloadTree.__init__(
            self,
            expr=items,
            item_class=item_class,
            )

    ### SPECIAL METHODS ###

    def __illustrate__(self, **kwargs):
        r'''Illustrates pitch-class tree.

        Returns LilyPond file.
        '''
        from abjad.tools import indicatortools
        from abjad.tools import lilypondfiletools
        from abjad.tools import markuptools
        from abjad.tools import scoretools
        from abjad.tools.topleveltools import override
        from scoremanager import idetools
        voice = scoretools.Voice()
        staff = scoretools.Staff([voice])
        score = scoretools.Score([staff])
        lilypond_file = lilypondfiletools.make_basic_lilypond_file(score)
        configuration = idetools.Configuration()
        stylesheet = os.path.join(
            configuration.abjad_stylesheets_directory,
            'rhythm-letter-16.ily',
            )
        lilypond_file.file_initial_user_includes.append(stylesheet)
        voice.consists_commands.append('Horizontal_bracket_engraver')
        leaf_list_stack = []
        self._bracket_inner_nodes(leaf_list_stack, self, voice)
        score.add_final_bar_line()
        override(score).bar_line.stencil = False
        override(score).flag.stencil = False
        override(score).stem.stencil = False
        override(score).text_script.staff_padding = 3
        override(score).time_signature.stencil = False
        if 'title' in kwargs:
            markup = markuptools.Markup(kwargs.get('title'))
            lilypond_file.header_block.title = markup
        if 'subtitle' in kwargs:
            markup = markuptools.Markup(kwargs.get('subtitle'))
            lilypond_file.header_block.subtitle = markup
        command = indicatortools.LilyPondCommand('accidentalStyle forget')
        lilypond_file.layout_block.items.append(command)
        return lilypond_file

    ### PRIVATE METHODS ###

    def _bracket_inner_nodes(self, leaf_list_stack, node, voice):
        from abjad.tools import durationtools
        from abjad.tools import markuptools
        from abjad.tools import scoretools
        from abjad.tools import spannertools
        from abjad.tools.topleveltools import attach
        if len(node):
            if node.level:
                leaf_list_stack.append([])
            for child_node in node:
                self._bracket_inner_nodes(
                    leaf_list_stack,
                    child_node,
                    voice,
                    )
            if node.level:
                bracket = spannertools.HorizontalBracketSpanner()
                attach(bracket, leaf_list_stack[-1])
                if node.level == 1:
                    node_index = node.parent.index(node)
                    level_one_first_leaf = leaf_list_stack[-1][0]
                    string = r'\bold {{ {} }}'.format(node_index)
                    markup = markuptools.Markup(string, Up)
                    attach(markup, level_one_first_leaf)
                leaf_list_stack.pop()
        elif node.payload:
            note = scoretools.Note(
                node.payload,
                durationtools.Duration(1, 8),
                )
            voice.append(note)
            for leaf_list in leaf_list_stack:
                leaf_list.append(note)


# -*- encoding: utf-8 -*-
from abjad.tools import sequencetools


def list_numbered_interval_numbers_pairwise(pitch_carriers, wrap=False):
    r'''List numbered interval numbers pairwise between `pitch_carriers`:

    ::

        >>> staff = Staff("c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")

    ..  doctest::

        >>> print(format(staff))
        \new Staff {
            c'8
            d'8
            e'8
            f'8
            g'8
            a'8
            b'8
            c''8
        }

    ::

        >>> pitchtools.list_numbered_interval_numbers_pairwise(
        ... staff)
        [2, 2, 1, 2, 2, 2, 1]

    ::

        >>> pitchtools.list_numbered_interval_numbers_pairwise(
        ... staff, wrap=True)
        [2, 2, 1, 2, 2, 2, 1, -12]

    ::

        >>> notes = [
        ...     Note("c'8"), Note("d'8"), Note("e'8"), Note("f'8"),
        ...     Note("g'8"), Note("a'8"), Note("b'8"), Note("c''8")]

    ::

        >>> notes.reverse()

    ::

        >>> pitchtools.list_numbered_interval_numbers_pairwise(
        ... notes)
        [-1, -2, -2, -2, -1, -2, -2]

    ::

        >>> pitchtools.list_numbered_interval_numbers_pairwise(
        ... notes, wrap=True)
        [-1, -2, -2, -2, -1, -2, -2, 12]

    When ``wrap = False`` do not return ``pitch_carriers[-1] - pitch_carriers[0]``
    as last in series.

    When ``wrap = True`` do return ``pitch_carriers[-1] - pitch_carriers[0]``
    as last in series.

    Returns list.
    '''
    from abjad.tools import pitchtools

    result = []

    if len(pitch_carriers) == 0:
        return result
    elif len(pitch_carriers) == 1:
        if pitchtools.Pitch.is_pitch_carrier(pitch_carriers[0]):
            return result
        else:
            message = 'must be pitch, not, note-head or chord.'
            raise TypeError(message)

    if wrap:
        pairs = sequencetools.iterate_sequence_nwise(
            pitch_carriers, wrapped=True)
    else:
        pairs = sequencetools.iterate_sequence_nwise(pitch_carriers)

    for first_carrier, second_carrier in pairs:
        first_pitch = pitchtools.get_named_pitch_from_pitch_carrier(
            first_carrier)
        second_pitch = pitchtools.get_named_pitch_from_pitch_carrier(
            second_carrier)
        signed_interval = \
            pitchtools.NumberedPitch(second_pitch).pitch_number - \
            pitchtools.NumberedPitch(first_pitch).pitch_number
        result.append(signed_interval)

    return result



# -*- encoding: utf-8 -*-
from abjad import *


def test_sievetools_Sieve___init___01():
    r'''Initialize sieve from rc instances and logical operator keyword.
    '''

    rcs = []
    rcs.append(sievetools.ResidueClass(6, 0))
    rcs.append(sievetools.ResidueClass(6, 1))
    rcs.append(sievetools.ResidueClass(6, 4))
    sieve = sievetools.Sieve(rcs, logical_operator='or')

    assert isinstance(sieve, sievetools.Sieve)
    assert sieve.rcs == rcs
    assert sieve.logical_operator == 'or'


def test_sievetools_Sieve___init___02():
    r'''Initialize sieve from other sieve instance.
    '''

    rcs = []
    rcs.append(sievetools.ResidueClass(6, 0))
    rcs.append(sievetools.ResidueClass(6, 1))
    rcs.append(sievetools.ResidueClass(6, 4))
    sieve = sievetools.Sieve(sievetools.Sieve(rcs, logical_operator='or'))

    assert isinstance(sieve, sievetools.Sieve)
    assert sieve.rcs == rcs
    assert sieve.logical_operator == 'or'


# -*- encoding: utf-8 -*-


def cumulative_sums(sequence, start=0):
    r'''Cumulative sums of `sequence`.

    ::

        >>> mathtools.cumulative_sums([1, 2, 3, 4, 5, 6, 7, 8], start=None)
        [1, 3, 6, 10, 15, 21, 28, 36]

    Returns list.
    '''

    if start is None:
        result = []
    else:
        result = [start]

    for element in sequence:
        if result:
            new = result[-1] + element
            result.append(new)
        else:
            result.append(element)

    return result


# -*- encoding: utf-8 -*-
from abjad import *


def test_markuptools_Markup___repr___01():

    markup = markuptools.Markup('foo bar')
    assert repr(markup) == "Markup(contents=('foo bar',))"


def test_markuptools_Markup___repr___02():

    markup = markuptools.Markup('foo bar', direction=Up)
    assert repr(markup) == "Markup(contents=('foo bar',), direction=Up)"


def test_markuptools_Markup___repr___03():

    markup = markuptools.Markup(contents=('foo', 'bar'), direction=Up)
    assert repr(markup) == "Markup(contents=('foo', 'bar'), direction=Up)"


# -*- encoding: utf-8 -*-
import os
import webbrowser
from abjad.tools import systemtools
from abjad.tools.documentationtools import AbjadAPIGenerator
from abjad.tools.developerscripttools.DeveloperScript import DeveloperScript


class BuildApiScript(DeveloperScript):
    r'''Builds the Abjad APIs.

    ..  shell::

        ajv api --help

    '''

    ### CLASS VARIABLES ###

    class ExperimentalAPIGenerator(AbjadAPIGenerator):
        r'''API generator for the experimental package.
        '''

        _api_title = 'Abjad Experimental API'

        @property
        def docs_api_index_path(self):
            from abjad import abjad_configuration
            return os.path.join(
                abjad_configuration.abjad_root_directory,
                'experimental',
                'docs',
                'source',
                'index.rst',
                )

        @property
        def path_definitions(self):
            from abjad import abjad_configuration
            tools_code_path = os.path.join(
                abjad_configuration.abjad_root_directory,
                'experimental',
                'tools',
                )
            tools_docs_path = os.path.join(
                abjad_configuration.abjad_root_directory,
                'experimental',
                'docs',
                'source',
                'tools',
                )
            tools_package_prefix = 'experimental.tools.'
            tools_triple = (
                tools_code_path,
                tools_docs_path,
                tools_package_prefix,
                )
            demos_code_path = os.path.join(
                abjad_configuration.abjad_root_directory,
                'experimental',
                'demos',
                )
            demos_docs_path = os.path.join(
                abjad_configuration.abjad_root_directory,
                'experimental',
                'docs',
                'source',
                'demos',
                )
            demos_package_prefix = 'experimental.demos.'
            demos_triple = (
                demos_code_path,
                demos_docs_path,
                demos_package_prefix,
                )
            all_triples = (tools_triple, demos_triple)
            return all_triples

        @property
        def root_package(self):
            return 'experimental'

        @property
        def tools_package_path_index(self):
            return 2

    class AbjadIDEAPIGenerator(AbjadAPIGenerator):
        r'''API generator for the Abjad IDE package.
        '''

        _api_title = 'Abjad IDE API'

        @property
        def docs_api_index_path(self):
            from abjad import abjad_configuration
            return os.path.join(
                abjad_configuration.abjad_root_directory,
                'scoremanager',
                'docs',
                'source',
                'index.rst',
                )

        @property
        def path_definitions(self):
            from abjad import abjad_configuration
            code_path = os.path.join(
                abjad_configuration.abjad_root_directory,
                'scoremanager',
                )
            docs_path = os.path.join(
                abjad_configuration.abjad_root_directory,
                'scoremanager',
                'docs',
                'source',
                )
            package_prefix = 'scoremanager.'
            triple = (code_path, docs_path, package_prefix)
            all_triples = (triple,)
            return all_triples

        @property
        def root_package(self):
            return 'scoremanager'

        @property
        def tools_package_path_index(self):
            return 1

    ### PUBLIC PROPERTIES ###

    @property
    def alias(self):
        r'''Alias of script.

        Returns ``'api'``.
        '''
        return 'api'

    @property
    def long_description(self):
        r'''Long description of script.

        Returns string or none.
        '''
        return None

    @property
    def scripting_group(self):
        r'''Scripting group of script.

        Returns none.
        '''
        return None

    @property
    def short_description(self):
        r'''Short description of script.

        Returns string.
        '''
        return 'Build the Abjad APIs.'

    @property
    def version(self):
        r'''Version of script.

        Returns float.
        '''
        return 1.0

    ### PRIVATE METHODS ###

    def _build_api(
        self,
        docs_directory=None,
        api_generator=None,
        api_title=None,
        api_format='html',
        clean=False,
        ):
        api_generator(verbose=True)
        print('Now building the {} {} docs ...'.format(
            api_title,
            api_format.upper(),
            ))
        print('')
        with systemtools.TemporaryDirectoryChange(docs_directory):
            if clean:
                print('Cleaning build directory ...')
                command = 'make clean'
                systemtools.IOManager.spawn_subprocess(command)
            if format == 'coverage':
                command = 'sphinx-build -b coverage {} {}'.format(
                    'source',
                    os.path.join('build', 'coverage'),
                    )
                systemtools.IOManager.spawn_subprocess(command)
            else:
                command = 'make {}'.format(api_format)
                systemtools.IOManager.spawn_subprocess(command)

    def _build_experimental_api(self, api_format='html', clean=False):
        from abjad import abjad_configuration
        api_generator = BuildApiScript.ExperimentalAPIGenerator()
        api_title = 'experimental'
        docs_directory = os.path.join(
            abjad_configuration.abjad_root_directory,
            'experimental',
            'docs',
            )
        self._build_api(
            api_generator=api_generator,
            api_title=api_title,
            api_format=api_format,
            clean=clean,
            docs_directory=docs_directory,
            )
        path = os.path.join(
            abjad_configuration.abjad_root_directory,
            'experimental',
            'docs',
            'build',
            'html',
            'index.html',
            )
        return path

    def _build_mainline_api(self, api_format='html', clean=False):
        from abjad import abjad_configuration
        api_generator = AbjadAPIGenerator()
        api_title = 'mainline'
        docs_directory = os.path.join(
            abjad_configuration.abjad_directory,
            'docs',
            )
        self._build_api(
            api_generator=api_generator,
            api_title=api_title,
            api_format=api_format,
            clean=clean,
            docs_directory=docs_directory,
            )
        path = os.path.join(
            abjad_configuration.abjad_root_directory,
            'abjad',
            'docs',
            'build',
            'html',
            'api',
            'index.html',
            )
        return path

    def _build_scoremanager_api(self, api_format='html', clean=False):
        from abjad import abjad_configuration
        api_generator = BuildApiScript.AbjadIDEAPIGenerator()
        api_title = 'Abjad IDE'
        docs_directory = os.path.join(
            abjad_configuration.abjad_root_directory,
            'scoremanager',
            'docs',
            )
        self._build_api(
            api_generator=api_generator,
            api_title=api_title,
            api_format=api_format,
            clean=clean,
            docs_directory=docs_directory,
            )
        path = os.path.join(
            abjad_configuration.abjad_root_directory,
            'scoremanager',
            'docs',
            'build',
            'html',
            'index.html',
            )
        return path

    ### PUBLIC METHODS ###

    def process_args(self, args):
        r'''Processes `args`.

        Returns none.
        '''
        api_format = args.format
        clean = args.clean
        paths = []
        if args.mainline:
            path = self._build_mainline_api(
                api_format=api_format,
                clean=clean,
                )
            paths.append(path)
        if args.experimental:
            path = self._build_experimental_api(
                api_format=api_format,
                clean=clean,
                )
            paths.append(path)
        if args.scoremanager:
            path = self._build_scoremanager_api(
                api_format=api_format,
                clean=clean,
                )
            paths.append(path)
        if api_format == 'html' and args.openinbrowser:
            for path in paths:
                if path.startswith('/'):
                    path = 'file://' + path
                webbrowser.open(path)

    def setup_argument_parser(self, parser):
        r'''Sets up argument `parser`.

        Returns none.
        '''
        parser.add_argument('-M', '--mainline',
            action='store_true',
            help='build the mainline API'
            )
        parser.add_argument('-X', '--experimental',
            action='store_true',
            help='build the experimental API'
            )
        parser.add_argument('-S', '--scoremanager',
            action='store_true',
            help='build the Abjad IDE API'
            )
        parser.add_argument('-C', '--clean',
            action='store_true',
            dest='clean',
            help='run "make clean" before building the api',
            )
        parser.add_argument('-O', '--open',
            action='store_true',
            dest='openinbrowser',
            help='open the docs in a web browser after building',
            )
        parser.add_argument('--format',
            choices=(
                'coverage',
                'html',
                'latex',
                'latexpdf',
                ),
            help='Sphinx builder to use',
            metavar='FORMAT',
            )
        parser.set_defaults(format='html')


# -*- encoding: utf-8 -*-


def iterate(expr):
    r'''iterates `expr`.

    ..  container:: example

        ::

            >>> staff = Staff("c'4 e'4 d'4 f'4")
            >>> show(staff) # doctest: +SKIP

        ::

            >>> notes = staff[-2:]
            >>> iterate(notes)
            IterationAgent(client=SliceSelection(Note("d'4"), Note("f'4")))

    Returns score iteration agent.
    '''
    from abjad.tools import agenttools
    return agenttools.IterationAgent(expr)


# -*- encoding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools import selectiontools


def make_leaves_from_talea(
    talea,
    talea_denominator,
    decrease_durations_monotonically=True,
    tie_rests=False,
    forbidden_written_duration=None,
    ):
    r'''Make leaves from `talea`.

    Interpret positive elements in `talea` as notes numerators.

    Interpret negative elements in `talea` as rests numerators.

    Set the pitch of all notes to middle C.

    ..  container:: example

        **Example 1.** Make leaves from talea:

        ::

            >>> leaves = scoretools.make_leaves_from_talea([3, -3, 5, -5], 16)
            >>> staff = Staff(leaves)
            >>> staff.context_name = 'RhythmicStaff'
            >>> time_signature = TimeSignature((4, 4))
            >>> attach(time_signature, staff)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print(format(staff))
            \new RhythmicStaff {
                \time 4/4
                c'8.
                r8.
                c'4 ~
                c'16
                r4
                r16
            }

    ..  container:: example

        **Example 2.** Increase durations monotonically:

        ::

            >>> leaves = scoretools.make_leaves_from_talea(
            ...     [3, -3, 5, -5], 16,
            ...     decrease_durations_monotonically=False)
            >>> staff = Staff(leaves)
            >>> staff.context_name = 'RhythmicStaff'
            >>> time_signature = TimeSignature((4, 4))
            >>> attach(time_signature, staff)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print(format(staff))
            \new RhythmicStaff {
                \time 4/4
                c'8.
                r8.
                c'16 ~
                c'4
                r16
                r4
            }

    ..  container:: example

        **Example 3.** Forbid written durations greater than or equal
        to a half note:

        ::

            >>> leaves = scoretools.make_leaves_from_talea(
            ...     [3, -3, 5, -5], 16,
            ...     forbidden_written_duration=Duration(1, 4))
            >>> staff = Staff(leaves)
            >>> staff.context_name = 'RhythmicStaff'
            >>> time_signature = TimeSignature((4, 4))
            >>> attach(time_signature, staff)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print(format(staff))
            \new RhythmicStaff {
                \time 4/4
                c'8.
                r8.
                c'8 ~
                c'8 ~
                c'16
                r8
                r8
                r16
            }

    Returns selection.
    '''
    from abjad.tools import scoretools

    assert all(x != 0 for x in talea), repr(talea)

    result = []
    for note_value in talea:
        if 0 < note_value:
            pitches = [0]
        else:
            pitches = [None]
        leaves = scoretools.make_leaves(
            pitches,
            [durationtools.Duration(abs(note_value), talea_denominator)],
            decrease_durations_monotonically=decrease_durations_monotonically,
            tie_rests=tie_rests,
            forbidden_written_duration=forbidden_written_duration,
            )
        result.extend(leaves)

    result = selectiontools.Selection(result)
    return result



# -*- encoding: utf-8 -*-
from abjad.tools.datastructuretools.TypedList import TypedList


class TimeSignatureInventory(TypedList):
    '''An ordered list of time signatures.

    ::

        >>> inventory = indicatortools.TimeSignatureInventory([(5, 8), (4, 4)])

    ::

        >>> inventory
        TimeSignatureInventory([TimeSignature((5, 8)), TimeSignature((4, 4))])

    ::

        >>> (5, 8) in inventory
        True

    ::

        >>> TimeSignature((4, 4)) in inventory
        True

    ::

        >>> (3, 4) in inventory
        False

    ::

        >>> show(inventory) # doctest: +SKIP

    Time signature inventories implement the list interface and are mutable.
    '''

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### SPECIAL METHODS ###

    def __illustrate__(self, format_specification=''):
        r'''Formats time signature inventory.

        ::

            >>> show(inventory) # doctest: +SKIP

        Returns LilyPond file.
        '''
        from abjad.tools import lilypondfiletools
        from abjad.tools import scoretools
        measures = scoretools.make_spacer_skip_measures(self)
        staff = scoretools.Staff(measures)
        staff.context_name = 'RhythmicStaff'
        score = scoretools.Score([staff])
        lilypond_file = lilypondfiletools.make_basic_lilypond_file(score)
        return lilypond_file

    ### PRIVATE PROPERTIES ###

    @property
    def _attribute_manifest(self):
        from abjad.tools import systemtools
        return systemtools.AttributeManifest()

    @property
    def _item_callable(self):
        from abjad.tools import indicatortools
        return indicatortools.TimeSignature

    @property
    def _one_line_menu_summary(self):
        return ', '.join([time_signature.pair for time_signature in self])


# -*- encoding: utf-8 -*-
import os


def play(expr):
    r'''Plays `expr`.

    ::

        >>> note = Note("c'4")

    ::

        >>> topleveltools.play(note) # doctest: +SKIP

    This input creates and opens a one-note MIDI file.

    Abjad outputs MIDI files of the format ``file_name.mid``
    under Windows.

    Abjad outputs MIDI files of the format ``file_name.midi``
    under other operating systems.

    Returns none.
    '''
    from abjad import abjad_configuration
    from abjad.tools import systemtools
    from abjad.tools import topleveltools
    assert '__illustrate__' in dir(expr)
    midi_file_path, abjad_formatting_time, lilypond_rendering_time = \
        topleveltools.persist(expr).as_midi()
    midi_player = abjad_configuration['midi_player']
    systemtools.IOManager.open_file(midi_file_path, midi_player)


# -*- encoding: utf-8 -*-
from .AbjadConfiguration import AbjadConfiguration
from .FilesystemState import FilesystemState
from .AttributeDetail import AttributeDetail
from .BenchmarkScoreMaker import BenchmarkScoreMaker
from .Configuration import Configuration
from .ForbidUpdate import ForbidUpdate
from .IOManager import IOManager
from .ImportManager import ImportManager
from .LilyPondFormatBundle import LilyPondFormatBundle
from .LilyPondFormatManager import LilyPondFormatManager
from .Memoize import Memoize
from .NullContextManager import NullContextManager
from .ProgressIndicator import ProgressIndicator
from .RedirectedStreams import RedirectedStreams
from .StorageFormatManager import StorageFormatManager
from .StorageFormatSpecification import StorageFormatSpecification
from .AttributeManifest import AttributeManifest
from .TestManager import TestManager
from .TemporaryDirectoryChange import TemporaryDirectoryChange
from .Timer import Timer
from .UpdateManager import UpdateManager
from .WellformednessManager import WellformednessManager
from .requires import requires

_documentation_section = 'internals'


# -*- encoding: utf-8 -*-
from abjad.tools import sequencetools
from abjad.tools.pitchtools.Vector import Vector


class IntervalClassVector(Vector):
    r'''An interval-class vector.

    ::

        >>> pitch_segment = pitchtools.PitchSegment(
        ...     items=[0, 11, 7, 4, 2, 9, 3, 8, 10, 1, 5, 6],
        ...     )
        >>> numbered_interval_class_vector = pitchtools.IntervalClassVector(
        ...     items=pitch_segment,
        ...     item_class=pitchtools.NumberedInversionEquivalentIntervalClass,
        ...     )
        >>> items = sorted(numbered_interval_class_vector.items())
        >>> for interval, count in items:
        ...     print(interval, count)
        ...
        1 12
        2 12
        3 12
        4 12
        5 12
        6 6

    '''

    ### INITIALIZER ###

    def __init__(self, items=None, item_class=None):
        from abjad.tools import pitchtools
        if isinstance(items, (
            pitchtools.PitchSegment,
            pitchtools.PitchSet,
            pitchtools.PitchClassSegment,
            pitchtools.PitchClassSet,
            )):
            intervals = []
            for first, second in \
                sequencetools.yield_all_unordered_pairs_of_sequence(
                    tuple(items)):
                intervals.append(second - first)
            items = intervals
        Vector.__init__(
            self,
            items=items,
            item_class=item_class,
            )

    ### PRIVATE PROPERTIES ###

    @property
    def _named_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.NamedIntervalClass

    @property
    def _numbered_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.NumberedIntervalClass

    @property
    def _parent_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.IntervalClass

    ### PUBLIC METHODS ###

    @classmethod
    def from_selection(
        cls,
        selection,
        item_class=None,
        ):
        r'''Makes interval-class vector from `selection`.

        Returns interval-class vector.
        '''
        from abjad.tools import pitchtools
        pitch_segment = pitchtools.PitchSegment.from_selection(selection)
        return cls(
            pitch_segment,
            item_class=item_class,
            )



# -*- encoding: utf-8 -*-
from abjad.tools import scoretools
from abjad.tools import spannertools
from abjad.tools.rhythmmakertools.RhythmMaker import RhythmMaker
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import detach
from abjad.tools.topleveltools import iterate
from abjad.tools.topleveltools import new


class NoteRhythmMaker(RhythmMaker):
    r'''Note rhythm-maker.

    ..  container:: example

        Makes notes equal to the duration of input divisions. Adds ties where
        necessary:

        ::

            >>> maker = rhythmmakertools.NoteRhythmMaker()

        ::

            >>> divisions = [(5, 8), (3, 8)]
            >>> music = maker(divisions)
            >>> lilypond_file = rhythmmakertools.make_lilypond_file(
            ...     music,
            ...     divisions,
            ...     )
            >>> show(lilypond_file) # doctest: +SKIP

        ..  doctest::

            >>> staff = maker._get_rhythmic_staff(lilypond_file)
            >>> f(staff)
            \new RhythmicStaff {
                {
                    \time 5/8
                    c'2 ~
                    c'8
                }
                {
                    \time 3/8
                    c'4.
                }
            }

    Usage follows the two-step configure-once / call-repeatedly pattern shown
    here.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    _class_name_abbreviation = 'N'

    _human_readable_class_name = 'note rhythm-maker'

    ### INITIALIZER ###

    def __init__(
        self,
        beam_specifier=None,
        duration_spelling_specifier=None,
        tie_specifier=None,
        ):
        RhythmMaker.__init__(
            self,
            beam_specifier=beam_specifier,
            duration_spelling_specifier=duration_spelling_specifier,
            tie_specifier=tie_specifier,
            )

    ### SPECIAL METHODS ###

    def __call__(self, divisions, seeds=None):
        r'''Calls note rhythm-maker on `divisions`.

        ..  container:: example

            ::

                >>> maker = rhythmmakertools.NoteRhythmMaker()
                >>> divisions = [(5, 8), (3, 8)]
                >>> result = maker(divisions)
                >>> for x in result:
                ...     x
                Selection(Note("c'2"), Note("c'8"))
                Selection(Note("c'4."),)

        Returns list of selections. Each selection holds one or more notes.
        '''
        return RhythmMaker.__call__(
            self,
            divisions,
            seeds=seeds,
            )

    def __eq__(self, arg):
        r'''True when `arg` is a note rhythm-maker with values of
        `beam_specifier`, `duration_spelling_specifier` and `tie_specifier`
        equal to those of this note rhythm-maker. Otherwise false.

        ..  container:: example

            ::

                >>> maker_1 = rhythmmakertools.NoteRhythmMaker()
                >>> tie_specifier = rhythmmakertools.TieSpecifier(
                ...     tie_across_divisions=True,
                ...     )
                >>> maker_2 = rhythmmakertools.NoteRhythmMaker(
                ...     tie_specifier=tie_specifier,
                ...     )

            ::

                >>> maker_1 == maker_1
                True
                >>> maker_1 == maker_2
                False
                >>> maker_2 == maker_1
                False
                >>> maker_2 == maker_2
                True

        Returns boolean.
        '''
        return RhythmMaker.__eq__(self, arg)

    def __format__(self, format_specification=''):
        r'''Formats note rhythm-maker.

        Set `format_specification` to `''` or `'storage'`.

        ..  container:: example

            ::

                >>> maker = rhythmmakertools.NoteRhythmMaker()
                >>> print(format(maker))
                rhythmmakertools.NoteRhythmMaker()

        Returns string.
        '''
        superclass = super(NoteRhythmMaker, self)
        return superclass.__format__(format_specification=format_specification)

    def __hash__(self):
        r'''Hashes note rhythm-maker.

        Required to be explicitely re-defined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(NoteRhythmMaker, self).__hash__()

    def __repr__(self):
        r'''Gets interpreter representation of note rhythm-maker.

        ..  container:: example

            ::

                >>> rhythmmakertools.NoteRhythmMaker()
                NoteRhythmMaker()

        Returns string.
        '''
        return RhythmMaker.__repr__(self)

    ### PRIVATE PROPERTIES ###

    @property
    def _attribute_manifest(self):
        from abjad.tools import rhythmmakertools
        from abjad.tools import systemtools
        return systemtools.AttributeManifest(
            systemtools.AttributeDetail(
                name='beam_specifier',
                command='bs',
                editor=rhythmmakertools.BeamSpecifier,
                ),
            systemtools.AttributeDetail(
                name='duration_spelling_specifier',
                command='dss',
                editor=rhythmmakertools.DurationSpellingSpecifier,
                ),
            systemtools.AttributeDetail(
                name='tie_specifier',
                command='ts',
                editor=rhythmmakertools.TieSpecifier,
                ),
            )

    ### PRIVATE METHODS ###

    def _make_music(self, duration_pairs, seeds):
        from abjad.tools import rhythmmakertools
        selections = []
        specifier = self.duration_spelling_specifier
        if specifier is None:
            specifier = rhythmmakertools.DurationSpellingSpecifier()
        for duration_pair in duration_pairs:
            selection = scoretools.make_leaves(
                pitches=0,
                durations=[duration_pair],
                decrease_durations_monotonically=\
                    specifier.decrease_durations_monotonically,
                forbidden_written_duration=\
                    specifier.forbidden_written_duration,
                )
            selections.append(selection)
        beam_specifier = self.beam_specifier
        if beam_specifier is None:
            beam_specifier = rhythmmakertools.BeamSpecifier()
        if beam_specifier.beam_divisions_together:
            for component in iterate(selections).by_class():
                detach(spannertools.Beam, component)
            beam = spannertools.MultipartBeam()
            leaves = iterate(selections).by_class(scoretools.Leaf)
            leaves = list(leaves)
            attach(beam, leaves)
        elif beam_specifier.beam_each_division:
            for selection in selections:
                beam = spannertools.MultipartBeam()
                attach(beam, selection)
        return selections

    ### PUBLIC PROPERTIES ###

    @property
    def duration_spelling_specifier(self):
        r'''Gets duration spelling specifier of note rhythm-maker.

        ..  container:: example

            Forbids notes with written duration greater than or equal to
            ``1/2`` of a whole note:

            ::

                >>> duration_spelling_specifier = \
                ...     rhythmmakertools.DurationSpellingSpecifier(
                ...     forbidden_written_duration=Duration(1, 2),
                ...     )
                >>> maker = rhythmmakertools.NoteRhythmMaker(
                ...     duration_spelling_specifier=duration_spelling_specifier,
                ...     )

            ::

                >>> divisions = [(5, 8), (3, 8)]
                >>> music = maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     music,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> staff = maker._get_rhythmic_staff(lilypond_file)
                >>> f(staff)
                \new RhythmicStaff {
                    {
                        \time 5/8
                        c'4 ~
                        c'4 ~
                        c'8
                    }
                    {
                        \time 3/8
                        c'4.
                    }
                }

        Returns duration spelling specifier or none.
        '''
        return RhythmMaker.duration_spelling_specifier.fget(self)

    ### PUBLIC METHODS ###

    def reverse(self):
        r'''Reverses note rhythm-maker.

        ..  container:: example

            ::

                >>> maker = rhythmmakertools.NoteRhythmMaker()
                >>> reversed_maker = maker.reverse()

            ::

                >>> print(format(reversed_maker))
                rhythmmakertools.NoteRhythmMaker(
                    duration_spelling_specifier=rhythmmakertools.DurationSpellingSpecifier(
                        decrease_durations_monotonically=False,
                        ),
                    )

            ::

                >>> divisions = [(5, 8), (3, 8)]
                >>> music = maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     music,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> staff = maker._get_rhythmic_staff(lilypond_file)
                >>> f(staff)
                \new RhythmicStaff {
                    {
                        \time 5/8
                        c'2 ~
                        c'8
                    }
                    {
                        \time 3/8
                        c'4.
                    }
                }

        Defined equal to copy of note rhythm-maker with
        `duration_spelling_specifier` reversed.

        Returns new note rhythm-maker.
        '''
        from abjad.tools import rhythmmakertools
        duration_spelling_specifier = self.duration_spelling_specifier
        if duration_spelling_specifier is None:
            default = rhythmmakertools.DurationSpellingSpecifier()
            duration_spelling_specifier = default
        duration_spelling_specifier = duration_spelling_specifier.reverse()
        arguments = {
            'duration_spelling_specifier': duration_spelling_specifier,
            }
        result = new(self, **arguments)
        return result


# -*- encoding: utf-8 -*-
from abjad import *


def test_rhythmmakertools_BurnishSpecifier_burnish_output_01():

    burnish_specifier = rhythmmakertools.BurnishSpecifier(
        burnish_output=True,
        lefts=(0,),
        middles=(-1,),
        rights=(0,),
        left_lengths=(1,),
        right_lengths=(1,),
        )

    talea = rhythmmakertools.Talea(
        counts=(1,),
        denominator=16,
        )

    maker = rhythmmakertools.TaleaRhythmMaker(
        talea=talea,
        burnish_specifier=burnish_specifier,
        extra_counts_per_division=(2,),
        )

    divisions = [(3, 16), (3, 8)]
    music = maker(divisions)

    music = sequencetools.flatten_sequence(music)
    measures = scoretools.make_spacer_skip_measures(divisions)
    staff = Staff(measures)
    mutate(staff).replace_measure_contents(music)

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 3/16
                \tweak #'text #tuplet-number::calc-fraction-text
                \times 3/5 {
                    c'16
                    r16
                    r16
                    r16
                    r16
                }
            }
            {
                \time 3/8
                \tweak #'text #tuplet-number::calc-fraction-text
                \times 3/4 {
                    r16
                    r16
                    r16
                    r16
                    r16
                    r16
                    r16
                    c'16
                }
            }
        }
        '''
        ), format(staff)


def test_rhythmmakertools_BurnishSpecifier_burnish_output_02():

    burnish_specifier = rhythmmakertools.BurnishSpecifier(
        burnish_output=True,
        lefts=(-1,),
        middles=(0,),
        rights=(-1,),
        left_lengths=(1,),
        right_lengths=(1,),
        )

    talea = rhythmmakertools.Talea(
        counts=(1,),
        denominator=4,
        )

    maker = rhythmmakertools.TaleaRhythmMaker(
        talea=talea,
        burnish_specifier=burnish_specifier,
        extra_counts_per_division=(2,),
        )

    divisions = [(3, 16), (3, 8)]
    music = maker(divisions)

    music = sequencetools.flatten_sequence(music)
    measures = scoretools.make_spacer_skip_measures(divisions)
    staff = Staff(measures)
    mutate(staff).replace_measure_contents(music)

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 3/16
                \tweak #'text #tuplet-number::calc-fraction-text
                \times 3/5 {
                    r4
                    c'16 ~
                }
            }
            {
                \time 3/8
                \tweak #'text #tuplet-number::calc-fraction-text
                \times 3/4 {
                    c'8.
                    c'4
                    r16
                }
            }
        }
        '''
        ), format(staff)


def test_rhythmmakertools_BurnishSpecifier_burnish_output_03():

    burnish_specifier = rhythmmakertools.BurnishSpecifier(
        burnish_output=True,
        lefts=(-1,),
        middles=(0,),
        rights=(-1,),
        left_lengths=(1,),
        right_lengths=(1,),
        )

    talea = rhythmmakertools.Talea(
        counts=(1, 2, 3),
        denominator=16,
        )

    maker = rhythmmakertools.TaleaRhythmMaker(
        talea=talea,
        burnish_specifier=burnish_specifier,
        extra_counts_per_division=(0, 2),
        split_divisions_by_counts=(9,),
        )

    divisions = [(3, 8), (4, 8)]
    music = maker(divisions)

    music = sequencetools.flatten_sequence(music)
    measures = scoretools.make_spacer_skip_measures(divisions)
    staff = Staff(measures)
    mutate(staff).replace_measure_contents(music)

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 3/8
                {
                    r16
                    c'8 [
                    c'8. ]
                }
            }
            {
                \time 4/8
                \tweak #'text #tuplet-number::calc-fraction-text
                \times 3/5 {
                    c'16 [
                    c'8
                    c'8 ~ ]
                }
                {
                    c'16 [
                    c'16
                    c'8 ]
                    r16
                }
            }
        }
        '''
        ), format(staff)


def test_rhythmmakertools_BurnishSpecifier_burnish_output_04():

    burnish_specifier = rhythmmakertools.BurnishSpecifier(
        burnish_output=True,
        lefts=(-1,),
        middles=(0,),
        rights=(-1,),
        left_lengths=(1,),
        right_lengths=(2,),
        )

    talea = rhythmmakertools.Talea(
        counts=(1,),
        denominator=8,
        )

    maker = rhythmmakertools.TaleaRhythmMaker(
        talea=talea,
        burnish_specifier=burnish_specifier,
        extra_counts_per_division=(),
        )

    divisions = [(8, 8)]
    music = maker(divisions)

    music = sequencetools.flatten_sequence(music)
    measures = scoretools.make_spacer_skip_measures(divisions)
    staff = Staff(measures)
    mutate(staff).replace_measure_contents(music)

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 8/8
                r8
                c'8 [
                c'8
                c'8
                c'8
                c'8 ]
                r8
                r8
            }
        }
        '''
        ), format(staff)


# -*- encoding: utf-8 -*-
from abjad.tools import markuptools
from abjad.tools import scoretools
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import iterate


def label_notes_in_expr_with_note_indices(expr, markup_direction=Down):
    r'''Label notes in `expr` with note indices:

    ::

        >>> staff = Staff("c'8 d'8 r8 r8 g'8 a'8 r8 c''8")

    ::

        >>> labeltools.label_notes_in_expr_with_note_indices(staff)

    ..  doctest::

        >>> print(format(staff))
        \new Staff {
            c'8 _ \markup { \small 0 }
            d'8 _ \markup { \small 1 }
            r8
            r8
            g'8 _ \markup { \small 2 }
            a'8 _ \markup { \small 3 }
            r8
            c''8 _ \markup { \small 4 }
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Returns none.
    '''

    for i, note in enumerate(iterate(expr).by_class(scoretools.Note)):
        label = r'\small %s' % i
        markup = markuptools.Markup(label, markup_direction)
        attach(markup, note)



# -*- encoding: utf-8 -*-


def upper_camel_case_to_snake_case(string):
    r'''Changes upper camel case `string` to snake-case.

    ..  container:: example:

        ::

            >>> string = 'KeySignature'

        ::

            >>> stringtools.upper_camel_case_to_snake_case(string)
            'key_signature'

    Returns string.
    '''

    words = []
    current_word = string[0].lower()
    for letter in string[1:]:
        if letter.isupper():
            words.append(current_word)
            current_word = letter.lower()
        else:
            current_word = current_word + letter
    words.append(current_word)
    result = '_'.join(words)
    return result